{ Assortment of support routines for Delphi programs }

unit DSupport;

interface

uses Windows,SysUtils,IniFiles,Classes,Grids,Forms,Version,Controls,MMSystem;

{Arithmetic and Transendental functions }
function Log(X: extended): extended;
function Power(X,Y:double): double;
function AntiLog(X: double): double;
function Sign(X: double): double;
function ArcTan2(Y,X:double) : double;
function Mag3DVector(X,Y,Z:double):double;
function ClipFraction(V:double;N:integer): double;
function FindMin(V1,V2: double): double;
function FindMax(V1,V2: double): double;
function LinInterpolate(V1,V2,D: double): double;
function NearestMultiple(V,N: double;Low: boolean): double;
function FloatMod(X,Y: extended): extended;
function DivMod(Numerator, Denominator: integer; var Remainder: integer): integer; overload;
function DivMod(Numerator, Denominator: Int64; var Remainder: Int64): Int64; overload;
function DivMod(Numerator, Denominator: double; var Remainder: double): double; overload;
function DivMod(Numerator, Denominator: extended; var Remainder: extended): integer; overload;
function ExtendSign(B: byte): integer;


{Number manipulation routines }
procedure RealFormat(L,R: integer);
function GetXPLRealStr(V: double): string;
function PadInteger(V: integer; Width: integer): string;
function ExtractNumber(Str: string; var Ptr1: integer;var Num: double): boolean;
function GetNumberValue(S: string): double;
function ValidDateString(D:string): boolean;
procedure FixDate(var M,D,Y: integer);
function IsValidFloatStr(S: string): boolean;
function UnroundedFloat(V: extended; Digits: integer): string;
function LeftTruncateInteger(Value,Digit: int64): int64;
function RightZeroInteger(Value,Digit: int64): int64;
function GetIntegerDigit(Value,Digit: int64): int64;
function NormalizeRange(V,Min,Max: integer): integer; overload;
function NormalizeRange(V,Min,Max: double): double; overload;


{Number I/O routines }
function IntegerIn(var IFile: TextFile): integer;
procedure RealOut(var OFile: TextFile; V: double);
function RealIn(var IFile: TextFile): double;


{Character and string functions}
function IsDigit(C: char): boolean;
function IsHexDigit(C: char): boolean;
function IsAlpha(C:char): boolean;
function IsAlphaNumeric(C:char): boolean;
function IsUpperCaseChar(C: char): boolean;
function IsLowerCaseChar(C: char): boolean;
function FindChar(const C: Char; const S: string; Start: integer): integer;
function FindLastOccurrence(Str: string; C: Char): integer;
function FindString(const Substr,S: string; Start: integer): integer;
function PosNoCase(SubString,S: string): integer;
function PosWildCard(SubStr,S: string; Wild: char): integer;
function PosAfterIndex(const SubStr,Str: string; var Index: integer): integer;
function PosWildNoCase(SubString,S: string;Wild:char): integer;
function WildCompare(S1,S2: string): boolean;
function ExactCompare(S1,S2: string): boolean;
function UpperChar(C: char): char;
function LowerChar(C: char): char;
function PadLeftString(Str: string; Count: integer; C: char): string;
function PadRightString(Str: string; Count: integer; C: char): string;
function GetTabStrRight(Str: string; Tab: integer): string;
function GetTabStrLeft(Str: string; Tab: integer): string;
function TabRightInteger(I,Tab: longint): string;
function TabLeftInteger(I,Tab: longint): string;
function ExtractToken(S :string;Sep: char; var P: integer): string;
function RemoveControls(var S: string): boolean;
function RemoveSpaces(const S: string): string;
function MatchCount(S1,S2: string; NoCase: boolean): integer;
procedure EatLeadingSpaces(var S: string);
procedure EatLeadingChars(var S: string; const C: char);
procedure EatLeadTrailChars(var S: string; const C: char);
procedure EatTrailingChars(var S: string; const C: char);
function EatTrailingDigits(S: string): string;
procedure WrapString(var S: string; BreakChar: char; Column: integer);
function FillString(C: char; N: integer): string;
function ExtractStringLine(var Source: string; var Destination: string; var Inx: integer): char;
function CountCharMatches(S: string; C: Char): integer;

{Character and string I/O}
procedure TabStringOut(var OFile: TextFile; Str: string; TabCount:integer);
procedure CRLFOut(var OFile: TextFile);
procedure SpaceOut(var F: TextFile; Num: integer);

{File Management Routines}
function IsFileOpen(var F): boolean;
function IsFileAssigned(var F): boolean;
function ValidateFilename(var S: string): boolean;
function GetFileSizeByName(const FileName: string): LongInt;
function FileDateTime(const FileName: string): System.TDateTime;
function GetFilePosition(var F: TextFile): Longint;
function ExtractParentDirName(Path: string): string;
function ExtractNamePart(FileName: string): string;

function CreateUniqueFilename(Path,Name,Ext: string): string;
procedure ChangeDefaultDir(Name: string);
function IsFileInUse(FName: string): boolean;
procedure FileCopy(const Source, Destin:String);

{Time/Degrees decoding routines}
procedure HourToHMSM(Hours: extended; var Hour,Min,Sec,MSec: integer);
function HourMinuteSecStr(Hours: extended; DoDegrees,DoMilliseconds: boolean; Digits: integer): string;
function DaysHourMinuteSecStr(Hours: extended; DoMilliseconds: boolean; Digits: integer): string;


{Logical routines}
function PickString(B: boolean; S1,S2: string): string;
function PickInteger(B: boolean; V1,V2: Integer): Integer;
function PickFloat(B: boolean; V1,V2: Extended): Extended;
function SwapIntegerArgs(Swap: boolean; var A1,A2: integer): boolean;
procedure SwapRectangles(var R1,R2: TRect);


{Bit handling routines}
function IsBitSet(Value,Bit: dword): boolean;
function AreAllBitClear(Value,Bits: dword): boolean;
function AreAllBitSet(Value,Bits: dword): boolean;
function ClearBits(Value,Bits: DWord): DWord;
function SetBits(Value,Bits: DWord): DWord;

{Misc. Routines}
procedure ParseComLine(var Switch,Name: string);
procedure SetInternational(Flag: boolean);
procedure SetInternationalFoot(V: boolean);
function FileToWinHandle(FileName: string; FullPath: boolean): HWND;
function ReadRealIni(Ini: TIniFile;const Section, Ident: string; Default: double): double;
procedure WriteRealIni(Ini: TIniFile;const Section, Ident: string; Value: double);
function ReadPointIni(Ini: TIniFile; const Section, Ident: string; Default: TPoint): TPoint;
procedure WritePointIni(Ini: TIniFile;const Section, Ident: string; Value: TPoint);
procedure DoTimeDelay(MS: dword);
procedure DelayProcessMessages(MS: dword);
procedure DoHiResDelay(US: int64);
function GetHiResTick: dword;
procedure SetHelpfilePath;
function GetBuildNumber: string;
procedure EncodeINIString(INIFile: TIniFile; const Section, Ident, Value: string);
function DecodeINIString(INIFile: TIniFile; const Section, Ident, Default: string): string;
function IsValidHexString(S: string): boolean;

{Graphics Routines }
procedure LineToLastPixel(H: HDC; X1,Y1,X2,Y2: integer);
function GetLastPixelPos(X1,Y1: integer; var X2,Y2: integer): boolean;

{Delphi VCL Support}
procedure ScaleGrid(Grid: TCustomGrid;M,D: integer);
procedure CenterScrollBox(SB: TScrollBox);


var LastChar: char; 		{Last character read by TextFile routines}

const FilenameMax=255;		{ Maximum characters in a filename }

const DesignRes = 96;		{Current design resolution}

const
 CtrlA= #$01;	CtrlB= #$02;	CtrlC= #$03;	CtrlD= #$04;
 CtrlE= #$05;	CtrlF= #$06;	CtrlG= #$07;	CtrlH= #$08;
 CtrlI= #$09;	CtrlJ= #$0A;	CtrlK= #$0B;	CtrlL= #$0C;
 CtrlM= #$0D;	CtrlN= #$0E;	CtrlO= #$0F;	CtrlP= #$10;
 CtrlQ= #$11;	CtrlR= #$12;	CtrlS= #$13;	CtrlT= #$14;
 CtrlU= #$15;	CtrlV= #$16;	CtrlW= #$17;	CtrlX= #$18;
 CtrlY= #$19;	CtrlZ= #$1A;


Const
 SP_CHAR=#$20;  ESC_CHAR=#$1B;  EOF_CHAR=#$1A;  FF_CHAR=#$0C;
 CR_CHAR=#$0D;  LF_CHAR=#$0A;   TAB_CHAR=#$09;  CRLF_CHAR=#$0D#$0A;
 Apost_Char = #$27;

{All these constants work by multiplying}

var Feet2Meters: double;	{feet to meters}
var Meters2Feet: double;	{Meters to Feet}
const Degree2Radian=PI/180;	{Degrees to Radians}
const Radian2Degree=180/PI;	{Radians to Degrees}

const HalfPi = PI/2;		{90 degress in radians}
const TwoPI = PI*2;		{360 degrees in radians}

implementation

var  FLeft,FRight: integer;	{Real format information }

{----------------- Logic Routines ---------------------------}


function PickString(B: boolean; S1,S2: string): string;
{Return S1 if B is true else S2}
begin
if B then Result:=S1 else Result:=S2;
end;


function PickInteger(B: boolean; V1,V2: Integer): Integer;
{Return V1 if B is true else V2}
begin
if B then Result:=V1 else Result:=V2;
end;


function PickFloat(B: boolean; V1,V2: Extended): Extended;
{Return V1 if B is true else V2}
begin
if B then Result:=V1 else Result:=V2;
end;



function SwapIntegerArgs(Swap: boolean; var A1,A2: integer): boolean;
{Swap integer arguments based Swap boolean}
var T: integer;
begin
Result:=Swap;
if not Swap then exit;
T:=A1; A1:=A2; A2:=T;
end;


procedure SwapRectangles(var R1,R2: TRect);
{Swap specified rectangles}
var T: TRect;
begin
T:=R1; R1:=R2; R2:=T;
end;



{----------------- Bit manipulation routines ----------------------------------}

function IsBitSet(Value,Bit: dword): boolean;
{Returns true if any of the specifed bits are ones}
begin
Result:=(Value and Bit)<>0;
end;


function AreAllBitSet(Value,Bits: dword): boolean;
{Returns true if all of the specifed bits are ones}
begin
Result:=(Value and Bits)=Bits;
end;



function AreAllBitClear(Value,Bits: dword): boolean;
{Returns true if all of the specifed bits are Zeros}
begin
Result:=(Value and Bits)=0;
end;




function SetBits(Value,Bits: DWord): DWord;
{Set specified  bits by "ORing" }
{so other bits are not changed}
begin
Result:=Value or Bits;
end;



function ClearBits(Value,Bits: DWord): DWord;
{Clear specified bits by "ANDing"}
{ so other bits are not changed}
begin
Result:=Value and not Bits;
end;


{----------------- Miscellanious Routines ---------------------------}

procedure SetInternationalFoot(V: boolean);
{Set international vs US survey foot}
begin
if V then Feet2Meters:=0.3048
else Feet2Meters:=0.304800609601;
Meters2Feet:=1/Feet2Meters;
end;


procedure SetInternational(Flag: boolean);
{Enable/Disable International Support }
var WinIni: TIniFile;
var Tmp: string;
begin
if Flag then
	begin
	WinIni:=TIniFile.Create('win.ini');
	try
	with WinIni do
	 begin
	 Tmp:=ReadString('intl','sDecimal','.');
	 DecimalSeparator:=Tmp[1];
	 end;
	finally WinIni.Free; end;
	end
else DecimalSeparator:='.';
end;



procedure ParseComLine(var Switch,Name: string);
{ Extract filename and all switches from command line }
var P: integer;
var I: Word;
begin
Name:='';
Switch:='';
for I := 1 to ParamCount do Name:=Name+ParamStr(I);

P:=Pos('/',Name);
while P>0 do
	begin
        if (P+1)<=Length(Name) then Switch:=Switch+Name[P+1];
        Delete(Name,P,2);
	P:=Pos('/',Name);
	end;
Switch:=UpperCase(Switch);
end;




function FileToWinHandle(FileName: string; FullPath:boolean): HWND;
{Converts filename to handle of running window }
var Wnd,hInst: HWND;
var ModuleName: array[0..255] of Char;
var S: string;
begin
Result:=0;
Wnd := GetDesktopWindow;
if Wnd=0 then exit;
Wnd := GetWindow(Wnd, GW_CHILD);
while Wnd<>0 do
	begin
	hInst:=GetWindowLong(Wnd,GWL_HINSTANCE);
 	GetModuleFileName(HInst, ModuleName, SizeOf(ModuleName));
	S := StrPas(ModuleName);
	if not FullPath then S:=ExtractFilename(S);

        if (GetWindow(Wnd,GW_OWNER)=0) and IsWindowVisible(Wnd) then
	 if (CompareText(FileName,S)=0) then
		begin
		Result:=Wnd;
		exit;
		end;
        Wnd := GetWindow(Wnd, GW_HWNDNEXT);
	end;
end;


function ReadRealIni(Ini: TIniFile;const Section, Ident: string; Default: double): double;
{ Read real from specified INI file}
var S: string;
var T: char;
begin
T:=DecimalSeparator;
DecimalSeparator:='.';
S:=Ini.ReadString(Section,Ident,FloatToStr(Default));
Result:=StrToFloat(S);
DecimalSeparator:=T;
end;


procedure WriteRealIni(Ini: TIniFile;const Section, Ident: string; Value: double);
{ Write real to specified INI file}
var T: char;
begin
T:=DecimalSeparator;
DecimalSeparator:='.';
Ini.WriteString(Section,Ident,FloatToStr(Value));
DecimalSeparator:=T;
end;


function ReadPointIni(Ini: TIniFile; const Section, Ident: string; Default: TPoint): TPoint;
{ Read TPoint from specified INI file}
begin
Result.X:=Ini.ReadInteger(Section,Ident+'X',Default.X);
Result.Y:=Ini.ReadInteger(Section,Ident+'Y',Default.Y);
end;


procedure WritePointIni(Ini: TIniFile;const Section, Ident: string; Value: TPoint);
{ Write TPoint to specified INI file}
begin
Ini.WriteInteger(Section,Ident+'X',Value.X);
Ini.WriteInteger(Section,Ident+'Y',Value.Y);
end;


procedure DoTimeDelay(MS: dword);
{ Delay specified number of miliseconds}
{ Resolution 1 ms in 95, 5 ms in NT}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do;
end;




procedure DelayProcessMessages(MS: dword);
{ Delay specified number of miliseconds}
{Process messages at the same time}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do Application.ProcessMessages;
end;


function GetHiResTick: dword;
{Returns high resolution ticks in usec}
var C,Freq: Int64;
begin
QueryPerformanceFrequency(Freq);
QueryPerformanceCounter(C);
C:=(C * 1000000) div Freq;
Result:=TULargeInteger(C).LowPart;
end;




procedure DoHiResDelay(US: int64);
{Do High resolutio delay in microseconds}
var Stop: int64;
begin
Stop:=GetHiResTick+US;
while GetHiResTick<Stop do;
end;



procedure SetHelpfilePath;
{Take the design-time help filename and add the path for the executable}
begin
Application.helpFile:=ExtractFilePath(Application.ExeName) + Application.helpFile;
end;


function GetBuildNumber: string;
{Get build number from version info}
var I: integer;
var VersionInfo: TVersionInfo;
begin
Result:='';
VersionInfo:=TVersionInfo.Create(nil);
Result:=VersionInfo.FileVersion;
if Pointer(Result) = nil then exit;
I:=Length(Result);
while (Result[I]<>'.') and (I>1) do Dec(I);
Delete(Result,1,I);
VersionInfo.Free;
end;


procedure EncodeINIString(INIFile: TIniFile; const Section, Ident, Value: string);
{Encode an ini string so it can contain control characters}
var HexString: string;
var I: integer;
begin
HexString:='';
for I:=1 to Length(Value) do HexString:=HexString+IntToHex(byte(Value[I]),2);
IniFile.WriteString(Section,Ident,HexString);
end;




function IsValidHexString(S: string): boolean;
{Checks all chars in string for valid hex}
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do
 if not IsHexDigit(S[I]) then exit;
Result:=True;
end;



function DecodeINIString(INIFile: TIniFile;const Section, Ident, Default: string): string;
{Decode an ini string that contains control characters}
var HexString: string;
var I: integer;
var BS: string[3];
begin

HexString:=IniFile.ReadString(Section,Ident,'');
if not IsValidHexString(HexString) then begin Result:=Default; exit; end
else Result:='';

for I:=0 to (Length(HexString) div 2)-1 do
	begin
	BS:='$'+Copy(HexString,I*2+1,2);
	Result:=Result+char(StrToInt(BS));
	end;
end;



{------------------- String Handling Routines -------------------------}

function IsDigit(C: char): boolean;
{ Return true if char is a digit }
begin
Result:= (C>='0') and (C<='9');
end;


function IsHexDigit(C: char): boolean;
{ Return true if char is a Hex digit }
begin
case C of
 '0'..'9','a'..'f','A'..'F': Result:=True;
 else Result:=False;
 end;
end;



function IsAlpha(C:char): boolean;
{ Return True If Ascii Alpha }
begin
Result:= ((C>='A')  and  (C<='Z'))  or  ((C>='a')  and  (C<='z'));
end;


function IsAlphaNumeric(C:char): boolean;
{ Return True If Ascii AlphaNumeric }
begin
Result:=(C>='0') and (C<='9') or
	((C>='A')  and  (C<='Z'))  or
	((C>='a')  and  (C<='z'));
end;


function IsUpperCaseChar(C: char): boolean;
{Returns true if character is upper case}
begin
Result:=(C >= 'A') and (C <='Z');
end;



function IsLowerCaseChar(C: char): boolean;
{Returns true if character is lower case}
begin
Result:=(C >= 'a') and (C <='z');
end;



function UpperChar(C: char): char;
{ Convert character to upper case }
begin
Result:=C;
if (C>='a') and (C<='z') then Result:=char(byte(Result)-$20);
end;


function LowerChar(C: char): char;
{ Convert character to lower case }
begin
Result:=C;
if (C>='A') and (C<='Z') then Result:=char(byte(Result)+$20);
end;



function FindChar(const C: Char; const S: string; Start: integer): integer;
{Find specified character starting at Start}
begin
for Result:=Start to Length(S) do if S[Result]=C then exit;
Result:=0;
end;


function FindLastOccurrence(Str: string; C: Char): integer;
{Find the last occurence of the specified Char}
begin
for Result:=Length(Str) downto 1 do if Str[Result]=C then exit;
Result:=0;
end;



function FindString(const Substr,S: string; Start: integer): integer;
{Find specified Substr in S starting at Start}
var I,I2: integer;
begin
Result:=0;
for I:=Start to Length(S) do
	begin
	if (I+Length(Substr))>Length(S) then exit;
	for I2:=1 to Length(Substr) do
		begin
		if Substr[I2]<>S[I+I2-1] then break;
		if I2=Length(Substr) then
			begin
			Result:=I;
			exit;
			end;
		end;
	end;
end;



function WildCompare(S1,S2: string): boolean;
{ Compare Strings, ? And * Are Wild Cards }
{ Wild Card String Must Be In S1 }
var P,L,L1,L2: Integer;
begin
L1:=Length(S1);
L2:=Length(S2);
if L1<L2 then L:=L1 else L:=L2;
for P:=1 to L do
	begin
	if S1[P]='*' then  begin Result:= True; exit; end;
	if (S1[P]<>S2[P]) and (S1[P]<>'?') then  begin Result:= False; exit; end;
	end;
if L2>L1 then begin Result:=False; exit; end;
if L1<>L2 then
 for P:=L+1 to L1 do
	begin
	if S1[P]='*' then  begin Result:= True; exit; end;
	if S1[P]<>'?' then  begin Result:= False; exit; end;
	end;
Result:=True;
end;



function ExactCompare(S1,S2: string): boolean;
{Compare strings, strings must be same length}
begin
if Length(S1)=Length(S2) then Result:=WildCompare(S1,S2)
else if S1[Length(S1)]='*' then Result:=WildCompare(S1,S2)
else Result:=False;
end;




function PosNoCase(SubString,S:string): integer;
{Locate String S in SubString, not case sensitive}
begin
Result:=Pos(UpperCase(SubString),UpperCase(S));
end;



function PosWildCard(SubStr,S: string; Wild: char): integer;
{Locate SubString in string S, with wild cards}
var P2,L1,L2: integer;

	function Match: boolean;
	var P1,Inx3: integer;
	begin
	Result:=False;
	for P1:=1 to L1 do
		begin
		Inx3:=P2 + P1 -1;
		if (Inx3>L2) or ((SubStr[P1]<>S[Inx3]) and (SubStr[P1]<>Wild)) then exit;
		end;
	Result:=True;
	end;

begin
Result:=0;
L1:=Length(SubStr);
L2:=Length(S);
if L1>L2 then exit;
for P2:=1 to L2 do
  if (Substr[1]=S[P2]) or (Substr[1]=Wild) then
    if Match then begin Result:=P2; exit; end;
end;



function PosWildNoCase(SubString,S: string; Wild: char): integer;
{Locate String S in SubString, not case sensitive, wild cards}
begin
Result:=PosWildCard(UpperCase(SubString),UpperCase(S),Wild);
end;



function PosAfterIndex(const SubStr,Str: string; var Index: integer): integer;
{Find the Position of SubStr in Str starting at Index}
{Index is left point one character past end of the target}
{If SubStr is not found, Result will be 0 and Index = Length+1}
var SPInx,SP,P: PChar;
begin
if Index<1 then Index:=1;
SPInx:=@Str[Index];
P:=StrPos(SPInx,PChar(SubStr));
if P<> nil then
	begin
	Result:=Integer(P)-Integer(PChar(Str))+1;
        Index:=Result + Length(SubStr);
	end
else
	begin
	Index:=Length(Str)+1;
	Result:=0;
	end;
end;


function  PadLeftString(Str: string; Count: integer; C: char): string;
{ Pad left side of string to specified length, with specified char}
var I : Integer;
begin
Result:='';
for I:=1 to Count-Length(Str) do Result:=Result+C;
Result:=Result+Str;
end;



function PadRightString(Str: string; Count: integer; C: char): string;
{ Pad right side of string to specified length, with specified char}
var I: Integer;
begin
Result:=Str;
for I:=1 to Count-Length(Str) do Result:=Result+C;
end;



function  GetTabStrRight(Str:string; Tab: integer): string;
{ Return String Right Justified }
begin
Result:=PadLeftString(Str,Tab,SP_Char);
end;



function GetTabStrLeft(Str: string; Tab: integer): string;
{ Return String Left Justified }
begin
Result:=PadRightString(Str,Tab,SP_Char);
end;



function TabRightInteger(I,Tab: longint):string;
{ Return rigth justified integer }
begin
Result:=GetTabStrRight(IntToStr(I),Tab);
end;


function TabLeftInteger(I,Tab: longint):string;
{ Return left justified integer }
begin
Result:=GetTabStrLeft(IntToStr(I),Tab);
end;


function ExtractToken(S: string; Sep: char; var P: integer): string;
{Extract token from S, starting at P up to but not including Sep}
{Terminates with P pointing past Sep or past end of string}
var C: char;
begin
Result:='';
while P<=Length(S) do
	begin
	C:=S[P]; Inc(P);
	if C=Sep then break
	else Result:=Result+C;
	end;
end;



function RemoveControls(var S: string): boolean;
{Remove control characters}
var P1,P2: integer;
begin
P2:=0;
for P1:=1 to Length(S) do
   if S[P1]>=#$20 then
	begin Inc(P2); S[P2]:=S[P1]; end;
Result:=P2<>Length(S);
SetLength(S,P2);
end;


function RemoveSpaces(const S: string): string;
{Remove space characters}
var P1: integer;
begin
Result:='';
for P1:=1 to Length(S) do if S[P1]<>#$20 then Result:=Result+S;
end;


procedure EatLeadingChars(var S: string; const C: char);
{Remove leading chars from a string}
begin
while S[1] = C do Delete(S,1,1);
end;



procedure EatTrailingChars(var S: string; const C: char);
{Remove trailing char from a string}
begin
while S[Length(S)] = C do Delete(S,Length(S),1);
end;



procedure EatLeadTrailChars(var S: string; const C: char);
{Remove trailing and leading char from a string}
begin
EatLeadingChars(S,C);
EatTrailingChars(S,C);
end;



function EatTrailingDigits(S: string): string;
{Remove all trailing numerical digits from string}
begin
Result:=S;
while IsDigit(Result[Length(Result)]) do Delete(Result,Length(Result),1);
end;



procedure EatLeadingSpaces(var S: string);
{Remove leading spaces from a string}
begin
while Pos(' ',S)=1 do Delete(S,1,1);
end;



function MatchCount(S1,S2: string; NoCase: boolean): integer;
{Count the number of matching characters in S1 and S2}
var P,L: integer;
begin
Result:=0;
if NoCase then begin S1:=UpperCase(S1); S2:=UpperCase(S2); end;
if Length(S1)<Length(S2) then L:=Length(S1) else L:=Length(S2);
for P:=1 to L do
 if S1[P]=S2[P] then Inc(Result)
 else break;
end;



procedure WrapString(var S: string; BreakChar: char; Column: integer);
{Wrap string at specified break char past specifed column}
var I: integer;
begin
I:=1;
while true do
	begin
	Inc(I,Column);
	while true do
		begin
		if I>Length(S) then exit;
		if S[I]=BreakChar then
			begin
			Delete(S,I,1);
			Insert(CRLF_Char,S,I);
			break;
			end;
		Inc(I);
		end;
	end;
end;



function FillString(C: char; N: integer): string;
{Fill string with N number of the specified character}
var I: integer;
begin
Result:='';
for I:=1 to N do Result:=Result + C;
end;



function ExtractStringLine(var Source: string; var Destination: string; var Inx: integer): char;
{Fast routine for extracting lines from strings}
{Call with index set to 1 and then repeat call until all strings extracted}
{Index is updated as lines are found }
var P,Start: PChar;
begin
P:=Pointer(Source);
Inc(P,Inx-1);
Start := P;
while not (P^ in [#0, LF_Char, CR_Char]) do Inc(P);
SetString(Destination, Start, P - Start);
Result:=P^;
if P^ = CR_Char then Inc(P);
if P^ = LF_Char then Inc(P);
Inx:=Inx + (integer(P) - integer(Start));
end;


function CountCharMatches(S: string; C: Char): integer;
{Return the number of time C is found in string}
var I: integer;
begin
Result:=0;
for I:=1 to Length(S) do
 if S[I]=C then Inc(Result);
end;

{ ------------------ Math Routines ------------------------------------}



function IntegerIn(var IFile: TextFile): integer;
{ Read integer from text file}
var S: String[128];
begin
S:='';
Read(IFile,LastChar);
while (LastChar<'0') or (LastChar>'9') do
	begin
	if Eof(IFile) then begin Result:=0; exit; end;
	if (LastChar = '-') or (LastChar = '+') then S:=S+LastChar
	else S:='';
	Read(IFile,LastChar);
	end;
while (LastChar>='0') and (LastChar<='9') do
	begin
	S:=S+LastChar;
	Read(IFile,LastChar);
	if Eof(IFile) then break;
	end;
Result:=StrToInt(S);
end;




function RealIn(var IFile: TextFile): double;
{ Read a real from the input chanel }
var S: String[128];
begin
S:='';
Read(IFile,LastChar);
while ((LastChar<'0') or (LastChar>'9')) and (LastChar<>'.') do
	begin
	if Eof(IFile) then begin Result:=0; exit; end;
	if (LastChar = '-') or (LastChar = '+') then S:=S+LastChar
	else S:='';
	Read(IFile,LastChar);
	end;
while (LastChar>='0') and (LastChar<='9') or
	(LastChar='.') or (LastChar='E') or (LastChar='e') or
	(LastChar='+') or (LastChar='-') do
	begin
	S:=S+LastChar;
	Read(IFile,LastChar);
	if Eof(IFile) then break;
	end;
Result:=StrToFloat(S);
end;



function ExtractNumber(Str: string; var Ptr1: integer; var Num: double): boolean;
{Extract a number from a string starting at Ptr1 }
var Len: integer;
var NStr: string;
var DecFlg: boolean;
begin
Result:=False;
DecFlg:=False;
NStr:='';
Len:=Length(Str);
if Len=0 then exit;
{ Look for leading +, -, . or digit}
while True do
	begin
	if Ptr1>Len then exit;
	case Str[Ptr1] of
	 '-': if (Length(NStr)=0) or (not DecFlg and (NStr[Length(NStr)]<>'-')) then NStr:=NStr+Str[Ptr1];
	 else if (Str[Ptr1]=DecimalSeparator) and not DecFlg  then
		begin
		NStr:=NStr+Str[Ptr1];
		DecFlg:=True;
		end
	 else if IsDigit(Str[Ptr1]) then break
	 else
		begin
		NStr:='';
		DecFlg:=False;
		end;
	 end;
	Ptr1:=Ptr1+1;
	end;
Result:=True;

while (Ptr1<=Len) and (IsDigit(Str[Ptr1]) or (Str[Ptr1]=DecimalSeparator)) do
	begin
	if Str[Ptr1]=DecimalSeparator then
		if DecFlg then break else DecFlg:=True;
	NStr:=NStr+Str[Ptr1]; Inc(Ptr1);
	end;

if (Str[Ptr1]='E') or (Str[Ptr1]='e') then
	begin
	NStr:=NStr+Str[Ptr1]; Inc(Ptr1);
	while (Ptr1<=Len) and (Str[Ptr1]='-') do
		begin
		if NStr[Length(NStr)]<>'-' then NStr:=NStr+Str[Ptr1];
		Inc(Ptr1);
		end;
	while (Ptr1<=Len) and (IsDigit(Str[Ptr1])) do
		begin
		NStr:=NStr+Str[Ptr1];
		Inc(Ptr1);
		end;
	end;

Num:=StrToFloat(NStr);
end;


function GetNumberValue(S: string): double;
{ Get real from string, return 0 if invalid string}
var P: integer;
var Tmp: double;
begin
P:=1;
if ExtractNumber(S,P,Tmp) then Result:=Tmp
else Result:=0;
end;



function IsValidFloatStr(S: string): boolean;
{ Verify string contains valid floating value }
var P:integer; Tmp: double;
begin
P:=1;
Result:=ExtractNumber(S,P,Tmp);
end;


procedure RealFormat(L,R: integer);
{Set real format }
begin
FLeft:=L; FRight:=R;
end;




procedure FloatToXPLStr(V: double; var FS: string);
{Convert real to XPL formated string }
var Len: integer;
var S: string;
begin
if FLeft<=0 then FS:=FloatToStrF(V,ffExponent,FRight+1,3)
else FS:=FloatToStrF(V,ffFixed,18,FRight);
S:='                  ';
for Len:=1 to Length(FS) do if FS[Len]='.' then break;
if Len>FLeft then SetLength(S,0)
else SetLength(S,FLeft-(Len-1));
FS:=S+FS;
end;



function UnroundedFloat(V: extended; Digits: integer): string;
{Returnes float as string without rounding past decimal point}
var Inx,DPI: integer;
begin
Result:=FloatToStrF(V,ffFixed,25,25);
if Pos('E',Result)>0 then exit;
Inx:=Pos('.',Result);
if Inx=0 then exit;
if Digits=0 then DPI:=0 else DPI:=1;
Delete(Result,Inx+Digits+DPI,Length(Result));
end;


function GetXPLRealStr(V: double): string;
{Get a real as an XPL formated number }
begin
FloatToXPLStr(V,Result);
end;


function PadInteger(V: integer; Width: integer): string;
{ Add leading zeros to fill string to width}
var I : Integer;
begin
Result:=IntToStr(V);
for I:=1 to Width - Length(Result) do Result:='0'+Result;
end;


procedure RealOut(var OFile: TextFile; V: double);
{Output a real as an XPL formated number }
var STmp: string;
begin
FloatToXPLStr(V,STmp);
Write(OFile,STmp);
end;


function Log(X: extended): extended;
{Calculate log 10 of X }
begin
Result:=Ln(X)/2.302585092994045680;
end;


function Sign(X: double) : double;
{ Return 1 or -1 depending on sign of X}
begin
if X < 0.0  then Result:= -1.0 else Result:= 1.0;
end;


function ArcTan2(Y,X:double) : double;
{ Return four quadrant ArcTangent }
begin
if X > 0.0 then Result:= ArcTan(Y/X)
else if X < 0.0  then  Result:= ArcTan(Y/X) + PI * Sign(Y)
else Result:= PI/2.0 * Sign(Y);
end;



function Power(X,Y:double): double;
{ X raise to the Y power }
begin
if X=0 then Result:=0
else
	begin
	Result:= Exp(Y * Ln(abs(X)));
	if X<0 then Result:=-Result;
	end;
end;



function AntiLog(X: double): double;
{Find anti log base 10}
begin
Result:=Exp(X * 2.30258509299404568401799145468436);
end;


function Mag3DVector(X,Y,Z:double): double;
{Calculate magnitude of a 3D vector}
begin
Result:=SQRT(X*X + Y*Y + Z*Z);
end;


function FindMin(V1,V2: double): double;
{Find minimum value}
begin
if V1<V2 then Result:=V1 else Result:=V2;
end;


function FindMax(V1,V2: double): double;
{Find maximum value}
begin
if V1>V2 then Result:=V1 else Result:=V2;
end;


function LinInterpolate(V1,V2,D: double): double;
{Linear interpolation}
begin
Result:=((V2-V1)*D)+V1;
end;



function NearestMultiple(V,N: double;Low: boolean): double;
{Return the nearest multiple to V, modulo N}
{Low controls whether the result is higher or lower then V}
begin
Result:=trunc(abs(V)/N)*N;
if (frac(V/N)<>0) and (not Low or (V<0)) then Result:=Result+N;
if V<0 then Result:=-Result;
end;



function FloatMod(X,Y: extended): extended;
{Return real remainder of X/Y }
begin
Result:= Frac(X/Y) * Y;
end;



function DivMod(Numerator, Denominator: integer; var Remainder: integer): integer; overload;
{Do 32-bit divide returning dividend and remainder}
begin
Result:=Numerator div Denominator;
Remainder:= Numerator mod Denominator;
end;



function DivMod(Numerator, Denominator: Int64; var Remainder: Int64): Int64; overload;
{Do 64-bit divide returning dividend and remainder}
begin
Result:=Numerator div Denominator;
Remainder:= Numerator mod Denominator;
end;


function DivMod(Numerator, Denominator: extended; var Remainder: extended): integer; overload;
{Do 32-bit divide returning dividend and remainder}
begin
Result:=Trunc(Numerator / Denominator);
Remainder:= FloatMod(Numerator,Denominator);
end;



function DivMod(Numerator, Denominator: double; var Remainder: double): double; overload;
{Do 32-bit divide returning dividend and remainder}
begin
Result:=Trunc(Numerator / Denominator);
Remainder:= FloatMod(Numerator,Denominator);
end;


function ExtendSign(B: byte): integer;
{Extend sign bit of byte to integer}
begin
if (B and $80)<>0 then Result:=$FFFFFF00 or B
else Result:=B;
end;


function ClipFraction(V:double;N:integer): double;
{ Clip fraction part to N digits }
var S: string;
begin
S:=FloatToStrF(V,ffFixed,18,N);
Result:=StrToFloat(S);
end;


function ValidDateString(D:string): boolean;
{ Test for a valid date string }
begin
Result:=True;
try StrToDate(D);
 except Result:=False;
 end;
end;



procedure FixDate(var M,D,Y: integer);
{Repair defective date}
const DaysPerMonth: array[1..12] of Integer =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
var C: integer;
begin
if Y<=0 then Y:=1;
if M<=0 then M:=1;
if M>12 then M:=12;
C:=DaysPerMonth[M];
if (M=2) and (Y mod 4 = 0) and
   ((Y mod 100 <> 0) or (Y mod 400 = 0)) then C:=C+1;
if D>C then D:=C;
if D<=0 then D:=1;
end;



function RightZeroInteger(Value,Digit: int64): int64;
{Return value with Digit number of digits zeroed}
var D,I: integer;
begin
D:=1;
for I:=1 to Digit do D:=D * 10;
Result:=(Value div D) * D;
end;




function LeftTruncateInteger(Value,Digit: int64): int64;
{Truncate all but Digits number of digits on left}
var T: integer;
begin
T:=RightZeroInteger(Value,Digit);
Result:=Value - T;
end;


function GetIntegerDigit(Value,Digit: int64): int64;
{Return one integer digit specified by Digit}
{0 = the ones digit}
var I: integer;
begin
Result:=Value;
for I:=1 to Digit do Result:=Result div 10;
Result:=Result - ((Result div 10) * 10);
end;



function NormalizeRange(V,Min,Max: integer): integer; overload;
{Normalize a number with in the specified range}
{Wraps numbers around if they exeed the specified Min and Max}
var D: integer;
begin
Result:=V;
D:=Max - Min;
while Result<Min do Result:=Result + D;
while Result>=Max do Result:=Result - D;
end;



function NormalizeRange(V,Min,Max: double): double; overload;
{Normalize a number with in the specified range}
{Wraps numbers around if they exeed the specified Min and Max}
var D: double;
begin
Result:=V;
D:=Max - Min;
while Result<Min do Result:=Result + D;
while Result>=Max do Result:=Result - D;
end;


{------------------- IO Routines --------------------------------------}

function ValidFileChar(C: char): boolean;
{ Return True if valid filename character}
var P: integer;
var V: string[20];
begin
V:='._^$~!#%&-{}()`''';
Result:=False;
     if IsAlpha(C) then Result:=True
else if IsDigit(C) then Result:=True
else for P:=1 to Length(V) do if V[P]=C then
	begin Result:=True; Exit; end;
end;



function ValidateFilename(var S: string): boolean;
{ Convert S to valid filename}
var P,C: integer;
begin
Result:=False;
if Length(S)<1 then exit;
C:=0;
for P:=Length(S) downto 1 do
 if S[P]='.' then
	begin
	Inc(C);
	if C>1 then S[P]:='-';
	end;
P:=1;
while P<Length(S) do
	begin
	if not ValidFileChar(S[P]) then Delete(S,P,1)
	else Inc(P);
	end;
Result:=True;
end;



function IsFileOpen(var F): boolean;
{Is delphi style file open }
var M: word;
begin
Result:=False;
if Pointer(F)=nil then exit;
M:=TFileRec(F).Mode;
Result:=(M=fmInput) or (M=fmOutput) or (M=fmInOut);
end;


function IsFileAssigned(var F): boolean;
{Is delphi style file assigned }
var M: word;
begin
M:=TFileRec(F).Mode;
Result:=(M=fmInput) or (M=fmOutput) or (M=fmInOut) or (M=fmClosed);
end;


function IsFileInUse(FName: string): boolean;
{Returns true if file is open by any application}
var H: integer;
begin
H:=FileOpen(FName,fmShareDenyRead or fmOpenRead);
Result:=H<0;
FileClose(H);
end;


procedure FileCopy(const Source, Destin:String);
{Copy file from Source to Destin}
var S, T: TFileStream;
begin
try Screen.Cursor:=crHourGlass;
S:=TFileStream.Create(Source,fmOpenRead);
try
T:=TFileStream.Create(Destin,fmOpenWrite or fmCreate);
try T.CopyFrom(S,S.Size);
finally T.Free; end;
finally S.Free; end;
finally Screen.Cursor:=crDefault; end;
end;


function GetFilePosition(var F:TextFile): Longint;
{Get position of file pointer for text file}
var H: word;
begin
H:=TTextRec(F).Handle;
Result:=FileSeek(H,0,1);
end;


function ExtractNamePart(FileName: string): string;
{Extract the name part of filename }
var I: Integer;
begin
Result:=ExtractFileName(FileName);
I := Length(Result);
while (I > 0) and not (Result[I] in ['.']) do Dec(I);
if (I > 0) then SetLength(Result,I-1);
end;


function ExtractParentDirName(Path: string): string;
{Extract the parent directory name from path}
var Inx: integer;
begin
Result:=ExtractFilePath(Path);
Inx:=Pos(':',Result);
if Inx>0 then Delete(Result,1,Inx);
if (Length(Result)>0) and (Result[Length(Result)]='\') then Delete(Result,Length(Result),1);
Inx:=FindLastOccurrence(Result,'\');
if Inx>0 then Delete(Result,1,Inx);
end;


function GetFileSizeByName(const FileName: string): LongInt;
{FInd the size of a file without opening it}
var SearchRec: TSearchRec;
begin
if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
else Result := -1;
end;



procedure ChangeDefaultDir(Name: string);
{Change default directory based on fully qualified filename}
var Path: string;
var Len: integer;
begin
Path:=ExtractFilePath(Name);
Len:=Length(Path);
if Len<1 then exit;
if Path[Len]='\' then Delete(Path,Len,1);
ChDir(Path);
end;


function FileDateTime(const FileName: string): System.TDateTime;
{Get the time/date for specified file }
begin
Result := FileDateToDateTime(FileAge(FileName));
end;




function CreateUniqueFilename(Path,Name,Ext: string): string;
{Create a Unique filane using Path, Name and Ext}
var Inx: integer;
begin
Inx:=1;
repeat
	begin
	Result:=Path+Name+IntToStr(Inx)+'.'+Ext;
	Inc(Inx);
	end
until not FileExists(Result);
end;



procedure  SpaceOut(var F:TextFile;Num:integer);
{ Output A Specified Number Of Spaces }
var C : Integer;
begin
for C:=1 to NUM do Write(F,SP_CHAR);
end;


procedure CRLFOut(var OFile: TextFile);
{ Send CRLF to file }
begin
Write(OFile, CR_CHAR, LF_CHAR);
end;


procedure  TabStringOut(var OFile: TextFile;Str:string; TabCount:integer);
{Output String With Tabulated Spaces }
var I : Integer;
var STmp: string;
begin
STmp:='';
for I:=1 to TabCount-Length(STR) do STmp:=STmp+' ';
Write(OFile,STmp,Str);
end; { Of Tabstr }


{------------------ Time/degree routines ----------------------------}


procedure HourToHMSM(Hours: extended; var Hour,Min,Sec,MSec: integer);
{Converts Time to Hours, Minutes, Sec, and MSec integers}
{Time is expressed as a real number of hours  }
var Rem: integer;
var Negative: boolean;
begin
Negative:=Hours<0;
if Negative then Hours:=-Hours;
Hour:=DivMod(round(Hours*3600000), 3600000, Rem);
Min:=DivMod(Rem, 60000, Rem);
Sec:=DivMod(Rem, 1000,MSec);
if Negative then Hour:=-Hour;
end;


function HourMinuteSecStr(Hours: extended; DoDegrees,DoMilliseconds: boolean; Digits: integer): string;
{Return hour, minute and second string.  If DoDegrees is true then return Degrees minutes seconds}
{If DoMilliseconds is true display Milliseconds. Digits controls decimals for seconds.}
var Hour,Min,Sec,MSec: integer;
var S1,S2,S3,DS: string;
begin
HourToHMSM(Hours,Hour,Min,Sec,MSec);
if DoDegrees then begin S1:=#$B0; S2:='"'; S3:=''''; end
else begin S1:=':'; S2:=':'; S3:=''; end;
Result:=IntToStr(Hour)+S1+PadInteger(Min,2)+S2+PadInteger(Sec,2);
if DoDegrees then
	begin
	DS:= FloatToStrF(MSec/1000,ffFixed,18,Digits);
	Delete(DS,1,1);
	if Digits>0 then Result:=Result + DS;
	Result:=Result+S3;
	end
else
	begin
	if DoMilliseconds then Result:=Result + ' ' + PadInteger(MSec,3)
	else if Digits>0 then Result:=Result + '.'+ FloatToStrF(MSec/1000,ffFixed,18,Digits);
	end;

end;


function DaysHourMinuteSecStr(Hours: extended; DoMilliseconds: boolean; Digits: integer): string;
{Return Days hour, minute and second string.}
{If DoMilliseconds is true display Milliseconds. Digits controls decimals for seconds.}
var Days: integer;
var Rem: extended;
begin
Days:=DivMod(Hours,24,Rem);
Result:=IntToStr(Days)+' Days '+HourMinuteSecStr(Rem,False,DoMilliseconds,Digits);
end;


{----------------- Graphics Routines --------------------------------}


function GetLastPixelPos(X1,Y1: integer; var X2,Y2: integer): boolean;
{Calculate the position of the last pixel for a line running from X1,Y1, to X2,Y2}
var DX,DY: integer;
begin
Result:=False;
DX:=X2-X1; DY:=Y2-Y1;
if (DX=0) and (DY=0) then exit;
if abs(DX)>abs(DY) then
	begin
	if DX=0 then Inc(Y2,round(DY div abs(DY)))
	else begin Inc(X2,round(DX div abs(DX))); Inc(Y2,round(DY div abs(DX))); end;
	end
else
	begin
	if DY=0 then Inc(X2,round(DX div abs(DX)))
	else begin Inc(Y2,round(DY div abs(DY))); Inc(X2,round(DX div abs(DY))); end;
	end;
Result:=True;
end;




procedure LineToLastPixel(H: HDC; X1,Y1,X2,Y2: integer);
{Draws a line from X1,Y1 to X2,Y2 - including the last pixel}
begin
if not GetLastPixelPos(X1,Y1,X2,Y2) then exit;
MoveToEx(H,X1,Y1,Nil);
LineTo(H,X2,Y2);
end;

{----------------- Delphi VCL Routines --------------------------------}


procedure ScaleGrid(Grid: TCustomGrid; M,D: integer);
{Scale rows and column sizes of a grid}
var I,M2,D2:integer;
begin
M2:=TForm(Grid.Owner).Canvas.TextHeight('0');
D2:=MulDiv(M2,D,M);
with TDrawGrid(Grid) do
 begin
 for I:=0 to ColCount-1 do
	begin
	ColWidths[I]:=MulDiv(ColWidths[I],M2,D2);
	//if (I mod 4)=0 then ColWidths[I]:=ColWidths[I]+1;
	end;
 for I:=0 to RowCount-1 do
	begin
	RowHeights[I]:=MulDiv(RowHeights[I],M2,D2);
	//if (I mod 4)=0 then RowHeights[I]:=RowHeights[I]+1;
	end;
 end;
end;


procedure CenterScrollBox(SB: TScrollBox);
{Center scrollbar positions}
begin
SB.HorzScrollBar.Position:=(SB.HorzScrollBar.Range-SB.ClientWidth) div 2;
SB.VertScrollBar.Position:=(SB.VertScrollBar.Range-SB.ClientHeight) div 2;
end;


initialization
SetInternationalFoot(True);
FLeft:=18;
FRight:=2;
end.

