unit HugeArry;

interface

uses Classes,SysUtils,WinProcs,WinTypes,Vector;


{------------------------------------------------------------}
{----------- Declaration for Huge array object --------------}
{------------------------------------------------------------}


type
 THugeArray = class(TObject)
 private
  Handle: THandle;
  ItemSize,MemSize: integer;
  MemPointer: Pointer;
  ExpandSize: integer;
  StartMax: integer;
  FMaxItems: integer;
 protected
  procedure SetMaxItems(V: integer);
 public
  ItemIndex: integer;
  Pointers: TList;
  constructor Create(Number,Size: LongInt);
  destructor Destroy; override;
  function GetItemPtr(Index: LongInt): Pointer;
  procedure ReAllocMem(NewMaxItems: LongInt);
  procedure ExpandArray;
  procedure SaveToStream(Stream: TStream);
  procedure LoadFromStream(Stream: TStream);
  procedure LoadFromFile(const FileName: string);
  procedure SaveToFile(const FileName: string);
  function AddPointer(var Item; Ptr: integer): Integer;
  function AddItem(var Item): boolean;
  function Add (var Item): integer;
  procedure InsertItem(var Item; Index: integer);
  procedure DeleteItem(Index: integer);
  procedure PutItem(var Item; Index: integer);
  procedure GetItem(var Item; Index: integer);
  procedure BlockMove(Source,Destin,Size: integer);
  procedure Exchange(Index1, Index2: Integer); virtual;
  procedure Sort(Compare: TListSortCompare); virtual;
  procedure Clear; dynamic;
  property Count: integer read ItemIndex write ItemIndex;
  property MaxItems: integer read FMaxItems write SetMaxItems;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Integer Array object ---------}
{------------------------------------------------------------}


type THugeInteger = class(THugeArray)
 private
  procedure Put(Index: integer; Item: integer);
  function Get(Index: integer): integer;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: integer): integer;
  function AddPointer(Item: integer; Ptr: integer): Integer;
  procedure Insert(Value: integer; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: integer read Get write Put; default;
 end;


{------------------------------------------------------------}
{---------- Declaration for Huge Byte Array object ----------}
{------------------------------------------------------------}

type THugeByte = class(THugeArray)
 private
  procedure Put(Index: integer; Item: Byte);
  function Get(Index: integer): Byte;
  protected
 public
  constructor Create(Number: integer);
  function Add(Item: Byte): integer;
  function AddPointer(Item: byte; Ptr: integer): Integer;
  procedure Insert(Value: byte; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: byte read Get write Put; default;
 end;

{------------------------------------------------------------}
{---------- Declaration for Huge char Array object ----------}
{------------------------------------------------------------}

type THugeChar = class(THugeArray)
 private
  procedure Put(Index: integer; const Item: char);
  function Get(Index: integer): char;
  protected
 public
  constructor Create(Number: integer);
  function Add(Item: char): integer;
  function AddPointer(Item: char; Ptr: integer): Integer;
  procedure Insert(Value: char; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: char read Get write Put; default;
 end;



{------------------------------------------------------------}
{-------- Declaration for Huge Boolean Array object ---------}
{------------------------------------------------------------}


type THugeBoolean = class(THugeArray)
 private
  procedure Put(Index: integer; Item: Boolean);
  function Get(Index: integer): boolean;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: boolean): integer;
  function AddPointer(Item: boolean; Ptr: integer): Integer;
  procedure Insert(Value: boolean; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: boolean read Get write Put; default;
 end;



{------------------------------------------------------------}
{-------- Declaration for Huge Longint Array object ---------}
{------------------------------------------------------------}


type THugeLongInt = class(THugeArray)
 private
  procedure Put(Index: integer; Item: LongInt);
  function Get(Index: integer): LongInt;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: integer): integer;
  function AddPointer(Item: longint; Ptr: integer): Integer;
  procedure Insert(Value: LongInt; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: longint read Get write Put; default;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Double  Array object ---------}
{------------------------------------------------------------}

type THugeDouble = class(THugeArray)
 private
  procedure Put(Index: integer; Item: double);
  function Get(Index: integer): double;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: double): integer;
  function AddPointer(Item: double; Ptr: integer): Integer;
  procedure Insert(Value: double; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: Double read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge Sinlge  Array object ---------}
{------------------------------------------------------------}

type THugeSingle = class(THugeArray)
 private
  procedure Put(Index: integer; Item: single);
  function Get(Index: integer): single;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: single): integer;
  function AddPointer(Item: single; Ptr: integer): Integer;
  procedure Insert(Value: single; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: Single read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge String  Array object ---------}
{------------------------------------------------------------}

type THugeString = class;

 THugeStringSortCompare = function(HugeString: THugeString; Index1, Index2: Integer): Integer;

 THugeString = class(THugeArray)
 private
  FCaseSensitive: boolean;
  procedure Put(Index: integer; Item: ShortString);
  function Get(Index: integer): ShortString;
  procedure QuickSort(L, R: Integer; SCompare: THugeStringSortCompare);
 protected
 public
  constructor Create(Number,Size: integer);
  destructor Destroy; override;
  procedure CustomSort(Compare: THugeStringSortCompare);
  procedure Exchange(Index1, Index2: Integer); override;
  function Add(Item: ShortString): integer;
  function AddPointer(Item: shortstring; Ptr: integer): Integer;
  function Compare(Target:ShortString; Index: integer): boolean;
  procedure Clear; override;
  procedure Insert(Value: shortstring; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: ShortString read Get write Put; default;
  property CaseSensitive: boolean read FCaseSensitive write FCaseSensitive;
 end;


{------------------------------------------------------------}
{-------- Declaration for 3DVector Array object -------------}
{------------------------------------------------------------}

type THuge3DVector = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T3DVector);
  function Get(Index: integer): T3DVector;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T3DVector): integer;
  function AddPointer(Item: T3DVector; Ptr: integer): Integer;
  procedure Insert(Value: T3DVector; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: T3DVector read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for 2DVector Array object -------------}
{------------------------------------------------------------}

type THuge2DVector = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T2DVector);
  function Get(Index: integer): T2DVector;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T2DVector): integer;
  function AddPointer(Item: T2DVector; Ptr: integer): Integer;
  procedure Insert(Value: T2DVector; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: T2DVector read Get write Put; default;
  procedure Rotate(Center: T2DVector; Angle: double);
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge 3DLine Array object ----------}
{------------------------------------------------------------}


type THuge3DLine = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T3DLine);
  function Get(Index: integer): T3DLine;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T3DLine): integer;
  function AddPointer(Item: T3DLine; Ptr: integer): Integer;
  procedure Insert(Value: T3DLine; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: T3DLine read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge 2DLine Array object ----------}
{------------------------------------------------------------}

type THuge2DLine = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T2DLine);
  function Get(Index: integer): T2DLine;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T2DLine): integer;
  function AddPointer(Item: T2DLine; Ptr: integer): Integer;
  procedure Insert(Value: T2DLine; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Rotate(Center: T2DVector; Angle: double);
  property Value[Index: integer]: T2DLine read Get write Put; default;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Point Array object -----------}
{------------------------------------------------------------}

type THugePoint = class(THugeArray)
 private
  procedure Put(Index: integer; Item: TPoint);
  function Get(Index: integer): TPoint;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: TPoint): integer;
  function AddPointer(Item: TPoint; Ptr: integer): Integer;
  procedure Insert(Value: TPoint; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: TPoint read Get write Put; default;
 procedure Rotate(Center: T2DVector; Angle: double);
 end;


implementation


{-------------------------------------------------------------------------}
{                  Implementation for huge array object                   }
{-------------------------------------------------------------------------}




constructor THugeArray.Create(Number,Size: integer);
{Create a huge array item}
var Available: integer;
var Buffer: TMemoryStatus;
begin
inherited Create;
Pointers:=TList.Create;
FMaxItems:=Number;
StartMax:=Number;
ItemSize:=Size;
MemSize:=FMaxItems*ItemSize;
if MemSize=0 then raise Exception.Create('Empty Array Allocation');
GlobalMemoryStatus(Buffer);

Available:=Buffer.dwAvailVirtual;
if Available<MemSize then
	begin
	MemSize:=Available;
	FMaxItems:=MemSize div ItemSize;
	end;
Handle:=GlobalAlloc(gmem_Moveable, MemSize);
if Handle=0 then OutOfMemoryError;
MemPointer := GlobalLock(Handle);
if MemPointer=nil then OutOfMemoryError;
ItemIndex:=0;
ExpandSize:=4;
end;



destructor THugeArray.Destroy;
{Destroy Huge array item}
begin
if Handle <> 0 then
	begin
	GlobalUnlock(Handle);
	GlobalFree(Handle);
	end;
{Make sure pointer creation does not recurse and blow the stack}
if Self.ClassType=THugeArray then Pointers.Free;
inherited Destroy;
end;


function THugeArray.GetItemPtr(Index: integer): Pointer;
{ Return pointer to item based on index }
begin
Result:=Pointer(Integer(MemPointer) + ItemSize*Index);
end;


procedure THugeArray.ReAllocMem(NewMaxItems: integer);
{ Reallocate memory for new number of items }
var NewHandle: THandle;
var TmpPtr: Pointer;
begin
if NewMaxItems =0 then raise Exception.Create('Empty Array Allocation');
NewHandle := GlobalReAlloc(Handle, NewMaxItems*ItemSize, GMEM_MOVEABLE);
if NewHandle = 0 then OutOfMemoryError;

try
TmpPtr := GlobalLock(NewHandle);
if TmpPtr = nil then OutOfMemoryError;
MemSize := GlobalSize(NewHandle);
FMaxItems :=MemSize div ItemSize;
Handle := NewHandle;
MemPointer:=TmpPtr;
except
 if Handle <> NewHandle then
	begin
	GlobalUnlock(NewHandle);
	GlobalFree(NewHandle);
	end;
 raise;
 end;
end;



procedure THugeArray.SetMaxItems(V: integer);
begin
if V<>FMaxItems then
	begin
	ReAllocMem(V);
	if Count>FMaxItems then Count:=FMaxItems;
	end;
end;



procedure THugeArray.ExpandArray;
{Expand array if necessary}
begin
if ItemIndex>=FMaxItems then
	begin
	if FMaxItems > 64 then ExpandSize := FMaxItems div 4
	else if FMaxItems > 8 then ExpandSize := 16
	else ExpandSize := 4;
	ReAllocMem(FMaxItems + ExpandSize);
	end;
end;



procedure THugeArray.SaveToStream(Stream: TStream);
begin
Stream.Write(MemPointer^,ItemSize * Count);
end;



procedure THugeArray.LoadFromStream(Stream: TStream);
var Items,Size: integer;
var ST: TStrings;
begin
Items:=Stream.Size div ItemSize;
ReAllocMem(Items);
Stream.Position:=0;
{Set the size this way in case the file}
{is not an even multiple of ItemSize}
Stream.Read(MemPointer^, Items * ItemSize);
Count:=Items;
end;



procedure THugeArray.LoadFromFile(const FileName: string);
var Stream: TStream;
begin
Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
try
LoadFromStream(Stream);
finally Stream.Free; end;
end;


procedure THugeArray.SaveToFile(const FileName: string);
var Stream: TStream;
begin
Stream := TFileStream.Create(FileName, fmCreate);
try
SaveToStream(Stream);
finally Stream.Free; end;
end;




function THugeArray.AddPointer(var Item; Ptr: integer): Integer;
{Add Item and Pointer to arrays}
begin
Result:=ItemIndex;
ExpandArray;
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*ItemIndex)^,ItemSize);
ItemIndex:=ItemIndex+1;
Pointers.Add(Pointer(Ptr));
end;



function THugeArray.AddItem(var Item): boolean;
{ Add item to the array, expanding memory if necessary }
begin
AddPointer(Item,0);
Result:=True;
end;


function THugeArray.Add (var Item): integer;
{ Add item to the array, expanding memory if necessary }
begin
Result:=AddPointer(Item,0);
end;


procedure THugeArray.PutItem(var Item; Index: integer);
{overwrite item at index}
begin
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*Index)^,ItemSize);
end;


procedure THugeArray.InsertItem(var Item; Index: integer);
{Insert item at index}
var Source,Destination: pointer;
var Size: integer;
begin
if (Index<0) or (Index> ItemIndex) then exit;
ExpandArray;
Source:=Pointer(Integer(MemPointer) + ItemSize*Index);
Destination:=Pointer(Integer(Source) + ItemSize);
Size:=(ItemIndex - Index) * ItemSize;
System.Move(Source^,Destination^,Size);
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*Index)^,ItemSize);
Inc(ItemIndex);
end;


procedure THugeArray.DeleteItem(Index: integer);
{Delete item at index}
var Source,Destination: pointer;
var Size: integer;
begin
if (Index<0) or (Index>=ItemIndex) then exit;
Source:=Pointer(Integer(MemPointer) + ItemSize*(Index+1));
Destination:=Pointer(Integer(Source) - ItemSize);
Size:=(ItemIndex - (Index+1)) * ItemSize;
System.Move(Source^,Destination^,Size);
Dec(ItemIndex);
end;


procedure THugeArray.GetItem(var Item; Index: integer);
{Get item at the index}
begin
System.Move(Pointer(Integer(MemPointer) + ItemSize*Index)^,Item,ItemSize);
end;



procedure THugeArray.BlockMove(Source,Destin,Size: integer);
{Do cross-segment huge block move}
begin
System.Move(GetItemPtr(Source)^,GetItemPtr(Destin)^,Size);
end;


procedure THugeArray.Clear;
{Clear data and reset to original state}
begin
ItemIndex:=0;
ReAllocMem(StartMax);
Pointers.Clear;
end;


procedure THugeArray.Exchange(Index1, Index2: Integer);
{Exchange the contents of pointed to by Index1 and Index2}
var B: byte;
var P1,P2: PByteArray;
var I: integer;
begin
P1:=GetItemPtr(Index1);
P2:=GetItemPtr(Index2);
for I:=0 to ItemSize-1 do
	begin
	B:=P1[I]; P1[I]:=P2[I]; P2[I]:=B;
	end;
Pointers.Exchange(Index1,Index2);
end;



procedure QuickSort(SortList: THugeArray; L, R: Integer; SCompare: TListSortCompare);
{Do quick sort on items held in HugeArray}
var I, J: Integer;
var P: Pointer;
begin
repeat
	begin
	I := L;
	J := R;
	P := SortList.GetItemPtr((L + R) shr 1);
	repeat
		begin
		while SCompare(SortList.GetItemPtr(I), P) < 0 do Inc(I);
		while SCompare(SortList.GetItemPtr(J), P) > 0 do Dec(J);
		if I <= J then
			begin
			SortList.Exchange(I,J);
			{This part added to make it work with pointers (not sure why)}
			if P = SortList.GetItemPtr(I) then P := SortList.GetItemPtr(J)
			else if P = SortList.GetItemPtr(J) then P := SortList.GetItemPtr(I);
			Inc(I);
			Dec(J);
			end;
		end
	until I > J;
	if L < J then QuickSort(SortList, L, J, SCompare);
	L := I;
	end
until I >= R;
end;


procedure THugeArray.Sort(Compare: TListSortCompare);
begin
if (Self<>nil) and (ItemIndex>0) then QuickSort(Self, 0, ItemIndex-1,Compare);
end;

{-------------------------------------------------------------------------}
{ Implementation for huge byte array object based on huge array object    }
{-------------------------------------------------------------------------}


constructor THugeByte.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(Byte));
end;


function THugeByte.Add(Item: byte): integer;
{Add item to byte array}
begin
Result:=inherited Add(Item);
end;


function THugeByte.AddPointer(Item: byte; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeByte.Put(Index: integer; Item: byte);
{Insert byte at index}
var PTR: ^Byte;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function THugeByte.Get(Index: integer): byte;
{Get byte at the index}
var PTR: ^Byte;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeByte.Insert(Value: byte; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeByte.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugeByte.Move(CurIndex, NewIndex: Integer);
var T: Byte;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{-------------------------------------------------------------------------}
{ Implementation for huge char array object based on huge array object    }
{-------------------------------------------------------------------------}


constructor THugeChar.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(char));
end;


function THugeChar.Add(Item: char): integer;
{Add item to byte array}
begin
Result:=inherited Add(Item);
end;

function THugeChar.AddPointer(Item: char; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;

procedure THugeChar.Put(Index: integer; const Item: char);
{Insert byte at index}
var PTR: ^char;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function THugeChar.Get(Index: integer): char;
{Get byte at the index}
var PTR: ^char;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;



procedure THugeChar.Insert(Value: char; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeChar.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeChar.Move(CurIndex, NewIndex: Integer);
var T: Char;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{-------------------------------------------------------------------------}
{ Implementation for huge boolean array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeBoolean.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(boolean));
end;


function THugeBoolean.Add(Item: boolean): integer;
{Add item to boolean array}
begin
Result:=inherited Add(Item);
end;


function THugeBoolean.AddPointer(Item: boolean; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeBoolean.Put(Index: integer; Item: boolean);
{Insert boolean at index}
var PTR: ^Boolean;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THugeBoolean.Get(Index: integer): boolean;
{Get boolean at the index}
var PTR: ^Boolean;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeBoolean.Insert(Value: boolean; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeBoolean.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeBoolean.Move(CurIndex, NewIndex: Integer);
var T: Boolean;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{-------------------------------------------------------------------------}
{ Implementation for huge integer array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeInteger.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(integer));
end;


function THugeInteger.Add(Item: integer): integer;
{Add item to integer array}
begin
Result:=inherited Add(Item);
end;


function THugeInteger.AddPointer(Item: integer; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeInteger.Put(Index: integer; Item: integer);
{Insert integer at index}
var PTR: ^Integer;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THugeInteger.Get(Index: integer): integer;
{Get integer at the index}
var PTR: ^Integer;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeInteger.Insert(Value: integer; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeInteger.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeInteger.Move(CurIndex, NewIndex: Integer);
var T: integer;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



{-------------------------------------------------------------------------}
{ Implementation for huge LongInt array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeLongInt.Create(Number: integer);
{Create a huge LongInt array}
begin
inherited Create(Number,SizeOf(LongInt));
end;


function THugeLongInt.Add(Item: LongInt): integer;
{Add item to LongInt array}
begin
Result:=inherited Add(Item);
end;


function THugeLongInt.AddPointer(Item: Integer; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeLongInt.Put(Index: integer; Item: LongInt);
{Insert LongInt at index}
var PTR: ^LongInt;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THugeLongInt.Get(Index: integer): LongInt;
{Get LongInt at the index}
var PTR: ^LongInt;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeLongInt.Insert(Value: LongInt; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeLongInt.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeLongInt.Move(CurIndex, NewIndex: Integer);
var T: longint;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



{------------------------------------------------------------------------}
{  Implementation for huge double array object based on huge array object}
{------------------------------------------------------------------------}


constructor THugeDouble.Create(Number: integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(double));
end;


function THugeDouble.Add(Item: double): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THugeDouble.AddPointer(Item: double; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeDouble.Put(Index: integer; Item: double);
{Insert Double at index}
var PTR: ^Double;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function THugeDouble.Get(Index: integer): Double;
{Get Double at the index}
var PTR: ^Double;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeDouble.Insert(Value: double; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeDouble.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeDouble.Move(CurIndex, NewIndex: Integer);
var T: double;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{-----------------------------------------------------------------------}
{ Implementation for huge single array object based on huge array object}
{-----------------------------------------------------------------------}

constructor THugeSingle.Create(Number: integer);
{Create a huge single array}
begin
inherited Create(Number,SizeOf(single));
end;


function THugeSingle.Add(Item: single): integer;
{Add item to single array}
begin
Result:=inherited Add(Item);
end;


function THugeSingle.AddPointer(Item: single; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugeSingle.Put(Index: integer; Item: single);
{Insert Single at index}
var PTR: ^Single;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THugeSingle.Get(Index: integer): Single;
{Get Single at the index}
var PTR: ^Single;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeSingle.Insert(Value: Single; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeSingle.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugeSingle.Move(CurIndex, NewIndex: Integer);
var T: single;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{-----------------------------------------------------------------------}
{ Implementation for huge string array object based on huge array object}
{-----------------------------------------------------------------------}


constructor THugeString.Create(Number,Size: integer);
{Create a huge string array}
begin
CaseSensitive:=True;
inherited Create(Number,Size+1);
end;



destructor THugeString.Destroy;
{Destroy Huge array item}
begin
inherited Destroy;
end;



procedure THugeString.Clear;
begin
inherited Clear;
end;


procedure THugeString.Exchange(Index1, Index2: Integer);
begin
inherited Exchange(Index1,Index2);
end;



procedure THugeString.QuickSort(L, R: Integer; SCompare: THugeStringSortCompare);
var I, J, P: Integer;
begin
repeat
	begin
	I := L; J := R;
	P := (L + R) shr 1;
	repeat
		begin
		while SCompare(Self, I, P) < 0 do Inc(I);
		while SCompare(Self, J, P) > 0 do Dec(J);
		if I <= J then
			begin
			Exchange(I, J);
			if P = I then P := J
			else if P = J then P := I;
			Inc(I);
			Dec(J);
			end;
		end
		until I > J;
	if L < J then QuickSort(L, J, SCompare);
	L := I;
	end
until I >= R;
end;



procedure THugeString.CustomSort(Compare: THugeStringSortCompare);
begin
QuickSort(0, ItemIndex - 1, Compare);
end;



function THugeString.AddPointer(Item: shortstring; Ptr: integer): Integer;
{Add string and Pointer}
begin
Result:=inherited AddPointer(Item,Ptr);
end;


function THugeString.Add(Item: ShortString): integer;
{Add item to string array}
begin
Result:=inherited Add(Item);
end;



procedure THugeString.Put(Index: integer; Item: ShortString);
{Insert String at index}
begin
System.Move(Item,GetItemPtr(Index)^,ItemSize);
end;


function THugeString.Get(Index: integer): ShortString;
{Get String at the index}
begin
System.Move(GetItemPtr(Index)^,Result,ItemSize);
end;


function THugeString.Compare(Target:ShortString; Index: integer): boolean;
{Compare Target to string at the index}
begin
if CaseSensitive then Result:=Target=Value[Index]
else Result:=CompareText(Target,Value[Index]) = 0;
end;


procedure THugeString.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeString.Insert(Value: shortstring; Index: integer);
begin
InsertItem(Value,Index);
end;



procedure THugeString.Move(CurIndex, NewIndex: Integer);
var T: shortstring;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;

{------------------------------------------------------------------------}
{  Implementation for huge Vector array object based on huge array object}
{------------------------------------------------------------------------}

constructor THuge3DVector.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T3DVector));
end;


function THuge3DVector.Add(Item: T3DVector): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge3DVector.AddPointer(Item: T3DVector; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THuge3DVector.Put(Index: integer; Item: T3DVector);
{Insert Double at index}
var PTR: ^T3DVector;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THuge3DVector.Get(Index: integer): T3DVector;
{Get Double at the index}
var PTR: ^T3DVector;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge3DVector.Insert(Value: T3DVector; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge3DVector.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge3DVector.Move(CurIndex, NewIndex: Integer);
var T: T3DVector;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;

{------------------------------------------------------------------------}
{  Implementation for 2DVector array object based on huge array object   }
{------------------------------------------------------------------------}

constructor THuge2DVector.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T2DVector));
end;


function THuge2DVector.Add(Item: T2DVector): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge2DVector.AddPointer(Item: T2DVector;  Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THuge2DVector.Put(Index: integer; Item: T2DVector);
{Insert Double at index}
var PTR: ^T2DVector;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THuge2DVector.Get(Index: integer): T2DVector;
{Get Double at the index}
var PTR: ^T2DVector;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge2DVector.Insert(Value: T2DVector; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge2DVector.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge2DVector.Move(CurIndex, NewIndex: Integer);
var T: T2DVector;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


procedure THuge2DVector.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var P: T2DVector;
begin
for I:=0 to Count-1 do
	begin
	P:=Self[I];
	RotatePoint2D(P,Center,Angle);
	Self[I]:=P;
	end;
end;




{------------------------------------------------------------------------}
{  Implementation for huge 3DLine array object based on huge array object}
{------------------------------------------------------------------------}

constructor THuge3DLine.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T3DLine));
end;


function THuge3DLine.Add(Item: T3DLine): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;



function THuge3DLine.AddPointer(Item: T3DLine; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THuge3DLine.Put(Index: integer; Item: T3DLine);
{Insert Double at index}
var PTR: ^T3DLine;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THuge3DLine.Get(Index: integer): T3DLine;
{Get Double at the index}
var PTR: ^T3DLine;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge3DLine.Insert(Value: T3DLine; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge3DLine.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge3DLine.Move(CurIndex, NewIndex: Integer);
var T: T3DLine;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;

{------------------------------------------------------------------------}
{  Implementation for huge Vector array object based on huge array object}
{------------------------------------------------------------------------}

constructor THugePoint.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(TPoint));
end;


function THugePoint.Add(Item: TPoint): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THugePoint.AddPointer(Item: TPoint; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THugePoint.Put(Index: integer; Item: TPoint);
{Insert Double at index}
var PTR: ^TPoint;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THugePoint.Get(Index: integer): TPoint;
{Get Double at the index}
var PTR: ^TPoint;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugePoint.Insert(Value: TPoint; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugePoint.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugePoint.Move(CurIndex, NewIndex: Integer);
var T: TPoint;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


procedure THugePoint.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var P: T2DVector;
begin
for I:=0 to Count-1 do
	begin
	P:=PointToVector2D(Self[I]);
	RotatePoint2D(P,Center,Angle);
	Self[I]:=Vector2DToPoint(P);
	end;
end;

{------------------------------------------------------------------------}
{ Implementation for huge 2D Line array object based on huge array object}
{------------------------------------------------------------------------}


constructor THuge2DLine.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T2DLine));
end;


function THuge2DLine.Add(Item: T2DLine): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge2DLine.AddPointer(Item: T2DLine; Ptr: integer): Integer;
begin
inherited AddPointer(Item,Ptr);
end;


procedure THuge2DLine.Put(Index: integer; Item: T2DLine);
{Insert Double at index}
var PTR: ^T2DLine;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;

function THuge2DLine.Get(Index: integer): T2DLine;
{Get Double at the index}
var PTR: ^T2DLine;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge2DLine.Insert(Value: T2DLine; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge2DLine.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THuge2DLine.Move(CurIndex, NewIndex: Integer);
var T: T2DLine;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;




procedure THuge2DLine.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var L: T2DLine;
begin
for I:=0 to Count-1 do
	begin
	L:=Self[I];
	RotateLine2D(L,Center,Angle);
	Self[I]:=L;
	end;
end;



end.
