unit Vector;

{Routines to handle vector math}

interface

uses Types,Graphics, SysUtils ,Math, DSupport;


{Vector, rectangle, line and triangle structures}

type T3DVector=packed record
  X,Y,Z: double;
  end;
type P3DVector = ^T3DVector;


type T2DVector=packed record
  X,Y: double;
  end;
type P2DVector = ^T2DVector;


type T2DRect = packed record
 case Integer of
  0: (Left, Top, Right, Bottom: double);
  1: (TopLeft, BottomRight: T2DVector);
  end;


type T2DLine = packed record
 P1,P2: T2DVector;
 end;


type T3DLine = packed record
 P1,P2: T3DVector;
 end;


type TTriangle = packed record
P1,P2,P3: T3DVector;
end;


{3D Plane object defined by azimuth and inclination vector}

type
 TPlane = class(TObject)
 private
   Con: T3DVector;
 protected
 public
   Azm,Dip: double;
   procedure SetPlane(Azm,Dip: double);
   function GetPlaneZ(X,Y: double): double;
   procedure GetAzmDip(var A,D: double);
   procedure SetEquation(V: T3DVector);
   function GetEquation: T3DVector;
   constructor Create;
  end;

{ 3D Functions }

function DotProduct3D(V1,V2: T3DVector): double;
function CrossProduct3D(V1,V2: T3DVector): T3DVector;
function RightCrossProduct3D(V1,V2: T3DVector): T3DVector;
function VectorABS3D(V: T3DVector): double;
function UnitVector3D(V: T3DVector): T3DVector;
function VectorAngle3D(V1,V2: T3DVector): double;
function ScalarAdd3D(V: T3DVector; S: double): T3DVector;
function ScalarSubtract3D(V: T3DVector; S: double): T3DVector;
function ScalarProduct3D(V: T3DVector; S: double): T3DVector;
function VectorMultiply3D(V1,V2: T3DVector): T3DVector;
function VectorSqrRoot3D(V: T3DVector): T3DVector;
function VectorFraction3D(V: T3DVector): T3DVector;
function VectorSquare3D(V: T3DVector): T3DVector;
function VectorDivide3D(V1,V2: T3DVector): T3DVector;
function ScalarDivide3D(V: T3DVector; S: double): T3DVector;
function VectorSubtract3D(V1,V2: T3DVector): T3DVector;
function VectorAdd3D(V1,V2: T3DVector): T3DVector;
function MakeVector3D(X,Y,Z: double): T3DVector; overload;
function MakeVector3D(V: T2DVector; Z: double): T3DVector; overload;
function MakeVector3DY(V: T2DVector): T3DVector;
function Vector3Dto2DZ(V: T3DVector; UseY: boolean): T2DVector;
function GetMinimumVector(V1,V2: T3DVector): T3DVector;
function GetMaximumVector(V1,V2: T3DVector): T3DVector;
function RangeCheckVectorMin(V: T3DVector; Min: double): T3DVector;  overload;
function RangeCheckVectorMin(V,Min: T3DVector): T3DVector; overload;

function RangeCheckVectorMax(V,Max: T3DVector): T3DVector; overload;
function RangeCheckVectorMax(V: T3DVector; Max: double): T3DVector;  overload;

procedure ExchangeVectors3D(var V1,V2: T3DVector);
function Vector3DToString(V: T3DVector; Digits: integer): string; overload;
function Vector3DToString(V: T3DVector; Digits: integer; Separator: char): string; overload;


{ 2D Functions }

function DotProduct2D(V1,V2: T2DVector): double;
function CrossProduct2D(V1,V2: T2DVector): T3DVector;
function VectorABS2D(V: T2DVector): double;
function UnitVector2D(V: T2DVector): T2DVector;
function NegateVector2D(V: T2DVector): T2DVector;
function VectorAngle2D(V1,V2: T2DVector): double;
function ScalarAdd2D(V: T2DVector; S: double): T2DVector;
function ScalarSubtract2D(V: T2DVector; S: double): T2DVector;
function ScalarProduct2D(V: T2DVector; S: double): T2DVector;
function ScalarDivide2D(V: T2DVector; S: double): T2DVector;
function VectorSubtract2D(V1,V2: T2DVector): T2DVector;
function VectorAdd2D(V1,V2: T2DVector): T2DVector;
function VectorDivide2D(V,S: T2DVector): T2DVector;
function VectorMultiply2D(V,S: T2DVector): T2DVector;
function MakeVector2D(X,Y: double): T2DVector;
function VectorSquare2D(V: T2DVector): T2DVector;
function VectorFraction2D(V: T2DVector): T2DVector;
function VectorSquareRoot2D(V: T2DVector): T2DVector;
function GetMinimumVector2D(V1,V2: T2DVector): T2DVector;
function GetMaximumVector2D(V1,V2: T2DVector): T2DVector;
function Vector2DToString(V: T2DVector; Digits: integer; Labeled: boolean): string;
function Vector2DToPoint(V: T2DVector): TPoint;
function PointToVector2D(P: TPoint): T2DVector;
procedure RotatePoint2D(var P: T2DVector; Center: T2DVector; Angle: double);
function PolarToVector2D(Point: T2DVector; Angle,Length: double): T2DVector;
function CompassToVector2D(Point: T2DVector; Angle,Length: double): T2DVector;


{Routines for 2D rectangles}

function Rect2DToRect(R: T2DRect): TRect;
procedure OffsetRect2D(var Rect: T2DRect; Offset: T2DVector);
procedure InflateRect2D(var Rect: T2DRect; Delta: T2DVector);
function MakeRect2D(X1,Y1,X2,Y2: double): T2DRect;  overload;
function MakeRect2D(TopLeft,BottomRight: T2DVector): T2DRect; overload;
procedure ExpandRect2D(var R: T2DRect; A: T2DVector);
procedure RotateRectangleCenter2D(var Rect: T2DRect; Center: T2DVector; Angle: double);
function RectToRect2D(Rect: TRect): T2DRect;
function MaxRotatedRect2D(Rect: T2DRect; Center: T2DVector; Angle: double): T2DRect;

{Routines for 2D Lines}

function Make2DLine(P1,P2: TPoint): T2DLine; overload;
function Make2DLine(P1,P2: T2DVector): T2DLine; overload;
function PolarToLine2D(Point: T2DVector; Angle,Length: double): T2DLine;
function CompassToLine2D(Point: T2DVector; Angle,Length: double): T2DLine;
procedure RotateLine2D(var Line: T2DLine; Center: T2DVector; Angle: double);
function IntersectLines2D(L1,L2: T2DLine; var Point: T2DVector): Boolean;
function LineLength2D(P1,P2: T2DVector) : double; overload;
function LineLength2D(L: T2DLine) : double; overload;
function LineCenter2D(L: T2DLine): T2DVector;
procedure ForceIntersect2D(var L1,L2: T2DLine);
function ExtendLine2D(L1: T2DLine; Len: double): T2DVector;
function RightTurn2D(L1, L2: T2DLine): Boolean;
procedure DrawLine2D(Canvas: TCanvas; L: T2DLine; C: TColor);


{ 3D Line Functions}

function GetDirectedVector3D(P1,P2: T3DVector; Length: double): T3DVector;
procedure TrimStartLine3D(var L: T3DLine; Amount: double);
procedure TrimEndLine3D(var L: T3DLine; Amount: double);
function MakeLine3D(P1,P2: T3DVector): T3DLine;



implementation


{------------ 2D Vector math routines --------------------}


function DotProduct2D(V1,V2: T2DVector): double;
{Do dot product of two vectors}
begin
Result:=(V1.X * V2.X) + (V1.Y * V2.Y);
end;


function VectorAdd2D(V1,V2: T2DVector): T2DVector;
{Add V1 and V2}
begin
Result.X:= V1.X + V2.X;
Result.Y:= V1.Y + V2.Y;
end;


function VectorSubtract2D(V1,V2: T2DVector): T2DVector;
{Subtract V2 from V1}
begin
Result.X:= V1.X - V2.X;
Result.Y:= V1.Y - V2.Y;
end;

function CrossProduct2D(V1,V2: T2DVector): T3DVector;
{Do crosss product of two vectors}
begin
Result.X:=0;
Result.Y:=0;
Result.Z:= ((V1.X * V2.Y) - (V1.Y * V2.X));
end;

function ScalarAdd2D(V: T2DVector; S: double): T2DVector;
{Add scalar to vector }
begin
Result.X:=V.X + S;
Result.Y:=V.Y + S;
end;



function ScalarSubtract2D(V: T2DVector; S: double): T2DVector;
{Subtract scalar from vector }
begin
Result.X:=V.X - S;
Result.Y:=V.Y - S;
end;


function ScalarProduct2D(V: T2DVector; S: double): T2DVector;
{Multiply vector by scalar}
begin
Result.X:=V.X * S;
Result.Y:=V.Y * S;
end;


function ScalarDivide2D(V: T2DVector; S: double): T2DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S;
Result.Y:=V.Y / S;
end;


function VectorDivide2D(V,S: T2DVector): T2DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S.X;
Result.Y:=V.Y / S.Y;
end;


function VectorMultiply2D(V,S: T2DVector): T2DVector;
{Multiply vector by vector}
begin
Result.X:=V.X * S.X;
Result.Y:=V.Y * S.Y;
end;


function VectorABS2D(V: T2DVector): double;
{Find ABS of vector}
begin
Result:=Sqrt(Sqr(V.X) + Sqr(V.Y));
end;


function UnitVector2D(V: T2DVector): T2DVector;
{Return unit vector}
var L: double;
begin
L:=VectorABS2D(V);
if L=0.0 then L:=1E-99;
Result.X:=V.X / L;
Result.Y:=V.Y / L;
end;


function NegateVector2D(V: T2DVector): T2DVector;
{Return negation of vector}
var L: double;
begin
Result.X:=-V.X;
Result.Y:=-V.Y;
end;


function VectorSquare2D(V: T2DVector): T2DVector;
{Return square of vector}
begin
Result:=VectorMultiply2D(V,V);
end;


function VectorSquareRoot2D(V: T2DVector): T2DVector;
{Return square of vector}
begin
Result.X:=Sqrt(V.X);
Result.Y:=Sqrt(V.Y);
end;


function VectorFraction2D(V: T2DVector): T2DVector;
{Return fractional part of vector components}
begin
Result.X:=Frac(V.X);
Result.Y:=Frac(V.Y);
end;


function VectorAngle2D(V1,V2: T2DVector): double;
{Return angle of two vectors}
var T,D: double;
begin
T:=VectorABS2D(V1)*VectorABS2D(V2);
if T=0 then Result:=0
else
	begin
	D:=DotProduct2D(V1,V2) / T;
	if abs(D)>1 then Result:=0
	else Result:=ArcCos(D);
	end;
end;



function MakeVector2D(X,Y: double): T2DVector;
{Create 2D Vector from X and Y}
begin
Result.X:=X;
Result.Y:=Y;
end;



function Vector2DToPoint(V: T2DVector): TPoint;
{Convert 2D Vector to a point}
begin
Result.X:=round(V.X);
Result.Y:=round(V.Y);
end;


function PointToVector2D(P: TPoint): T2DVector;
{Convertpoint to a 2D Vector }
begin
Result.X:=P.X;
Result.Y:=P.Y;
end;


function Vector3Dto2DZ(V: T3DVector; UseY: boolean): T2DVector;
{Convert 3D coordinate to 2D}
begin
Result.X:=V.X;
if UseY then Result.Y:=V.Y else Result.Y:=V.Z;
end;



function GetMinimumVector2D(V1,V2: T2DVector): T2DVector;
{Return minimum components of two vectors}
begin
Result.X:=Min(V1.X,V2.X);
Result.Y:=Min(V1.Y,V2.Y);
end;




function GetMaximumVector2D(V1,V2: T2DVector): T2DVector;
{Return minimum components of two vectors}
begin
Result.X:=Max(V1.X,V2.X);
Result.Y:=Max(V1.Y,V2.Y);
end;



function Vector2DToString(V: T2DVector; Digits: integer; Labeled: boolean): string;
{Return 3d vector as string. Digits past decimal in Digits}
var S1,S2: string;
begin
if Labeled then begin S1:='X: '; S2:=' Y: '; end
else begin S1:=''; S2:=' , '; end;
Result:=S1+FloatToStrF(V.X,ffFixed,18,Digits)+
        S2+FloatToStrF(V.Y,ffFixed,18,Digits);
end;




procedure RotatePoint2D(var P: T2DVector; Center: T2DVector; Angle: double);
{ Rotate point P around Center }
var D: T2DVector;
var SinTheta,CosTheta : double;
begin
CosTheta:=cos(Angle * Degree2Radian);
SinTheta:=sin(Angle * Degree2Radian);
D:=VectorSubtract2D(P,Center);
P.Y:=(SinTheta * -D.X) + (CosTheta * D.Y) + Center.Y;
P.X:=(CosTheta * D.X) + (SinTheta * D.Y) + Center.X;
end;



function PolarToVector2D(Point: T2DVector; Angle,Length: double): T2DVector;
{Convert point length and angle to vector}
begin
Angle:=Angle * Degree2Radian;
Result.X:=Length * Cos(Angle);
Result.Y:=Length * Sin(Angle);
Result:=VectorAdd2D(Result,Point);
end;



function CompassToVector2D(Point: T2DVector; Angle,Length: double): T2DVector;
{Convert point length and angle to vector}
{Uses compass angle where 0 degrees = north or up}
begin
Angle:=Angle + 90;
Result:=PolarToVector2D(Point,Angle,Length);
end;




{----------------- Routines for 2D Lines -------------------------------------}

function Make2DLine(P1,P2: TPoint): T2DLine; overload;
{Convert Points to 2D lines}
begin
Result.P1:=PointToVector2D(P1);
Result.P2:=PointToVector2D(P2);
end;


function Make2DLine(P1,P2: T2DVector): T2DLine; overload;
{Convert 2D Vectors to 2D lines}
begin
Result.P1:=P1;
Result.P2:=P2;
end;




procedure RotateLine2D(var Line: T2DLine; Center: T2DVector; Angle: double);
{ Rotate Line around Center }
begin
RotatePoint2D(Line.P1,Center,Angle);
RotatePoint2D(Line.P2,Center,Angle);
end;



function PolarToLine2D(Point: T2DVector; Angle,Length: double): T2DLine;
{Convert point length and angle to 2D Line}
begin
Result.P1:=Point;
Result.P2:=PolarToVector2D(Point,Angle,Length);
end;



function CompassToLine2D(Point: T2DVector; Angle,Length: double): T2DLine;
{Convert point length and angle to vector}
{Uses compass angle where 0 degrees = north or up}
begin
Angle:=Angle + 90;
Result:=PolarToLine2D(Point,Angle,Length);
end;



function IntersectLines2D(L1,L2: T2DLine; var Point: T2DVector): Boolean;
{ Return the intersection of two lines at Point, returns false if parallel  }
var A1,B1,C1,A2,B2,C2,N : double;
begin
A1:=L1.P2.Y - L1.P1.Y;
B1:=L1.P1.X - L1.P2.X;
A2:=L2.P2.Y - L2.P1.Y;
B2:=L2.P1.X - L2.P2.X;


C1:=(L1.P2.Y-L1.P1.Y) * L1.P1.X-(L1.P2.X-L1.P1.X) * L1.P1.Y;
C2:=(L2.P2.Y-L2.P1.Y) * L2.P1.X-(L2.P2.X-L2.P1.X) * L2.P1.Y;
N:= (A1 *B2 - A2 *B1);
if N=0 then begin Point:=L1.P2; Result:=False; exit; end;

Point.X := (B2 * C1 - B1 * C2) / N;
Point.Y := (A1 * C2 - A2 * C1) / N;
Result:=True;
end;




function LineLength2D(P1,P2: T2DVector) : double; overload;
{  Find length of a line defined by P1 and P2  }
var D: T2DVector;
begin
Result:=VectorABS2D(VectorSubtract2D(P2,P1));
end;



function LineLength2D(L: T2DLine) : double; overload;
{  Find length of a line defined by P1 and P2  }
begin
LineLength2D(L.P2,L.P1);
end;




Function LineCenter2D(L: T2DLine): T2DVector;
{  Find the center of line L  }
begin
Result.X := ((L.P2.X - L.P1.X)/2) + L.P1.X;
Result.Y := ((L.P2.Y - L.P1.Y)/2) + L.P1.Y;
end;



procedure ForceIntersect2D(var L1,L2: T2DLine);
{  Extend lines until they intersect  }
var Point: T2DVector;
begin
if  not IntersectLines2D(L1,L2,Point) then exit;
if LineLength2D(L1.P2,Point) < LineLength2D(L1.P1,L1.P2) then
	begin
	L1.P2:=Point;
	L2.P1 := Point;
	end;
end;



function ExtendLine2D(L1: T2DLine; Len: double): T2DVector;
{  Return a point that extends line L1 by Len  }
var Len1,UX,UY : double;
begin
Len1 := LineLength2D(L1)+1E-9;
UX := (L1.P2.X - L1.P1.X)/Len1;
UY := (L1.P2.Y - L1.P1.Y)/Len1;
Result.X := L1.P2.X +(UX *Len);
Result.Y := L1.P2.Y+(UY *Len);
end;



procedure DrawLine2D(Canvas: TCanvas; L: T2DLine; C: TColor);
{Draw Line on specified canvas}
begin
Canvas.Pen.Color:=C;
Canvas.MoveTo(Trunc(L.P1.X),Trunc(L.P1.Y));
Canvas.LineTo(Trunc(L.P2.X),Trunc(L.P2.Y));
end;



function RightTurn2D(L1, L2: T2DLine): Boolean;
{  return true if angle A2 goes right from A1 }
var T,A1,A2 : double;
begin
A1 := ArcTan2(L1.P2.X - L1.P1.X,L1.P2.Y - L1.P1.Y - 1E-6);
A2 := ArcTan2(L2.P2.X - L2.P1.X,L2.P2.Y - L2.P1.Y - 1E-6);

T := A2-A1;
if T>PI then T:=T-TwoPI else if T < -PI then T:=T+TwoPI;
Result:=(T>=0);
end;

{---------------- Routines for 2D rectangles -------------------}


function Rect2DToRect(R: T2DRect): TRect;
{Convert 2D rectangle to Windows integer rectangle}
begin
Result.TopLeft:=Vector2DToPoint(R.TopLeft);
Result.BottomRight:=Vector2DToPoint(R.BottomRight);
end;


function MakeRect2D(X1,Y1,X2,Y2: double): T2DRect;  overload;
{Create rectangle from indvidual points}
begin
Result.Left:=X1;
Result.Top:=Y1;
Result.Right:=X2;
Result.Bottom:=Y2;
end;



function MakeRect2D(TopLeft,BottomRight: T2DVector): T2DRect; overload;
{Make rectangle from 2D vertices}
begin
Result.TopLeft:=TopLeft;
Result.BottomRight:=BottomRight;
end;


function RectToRect2D(Rect: TRect): T2DRect;
{Convert integer rectangle to real rectangle}
begin
Result.Left:=Rect.Left;
Result.Right:=Rect.Right;
Result.Top:=Rect.Top;
Result.Bottom:=Rect.Bottom;
end;


procedure OffsetRect2D(var Rect: T2DRect; Offset: T2DVector);
{Move rectangle by specified offset}
begin
Rect.TopLeft:=VectorAdd2D(Rect.TopLeft,Offset);
Rect.BottomRight:=VectorAdd2D(Rect.BottomRight,Offset);
end;



procedure InflateRect2D(var Rect: T2DRect; Delta: T2DVector);
{Expand retangle by absolute amount specified in Delta}
begin
Rect.TopLeft:=VectorSubtract2D(Rect.TopLeft,Delta);
Rect.BottomRight:=VectorAdd2D(Rect.BottomRight,Delta);
end;


procedure ExpandRect2D(var R: T2DRect; A: T2DVector);
{Expand rectangle by percentage specified in A}
var D,N,X: T2DVector;
begin
D:=VectorSubtract2D(R.BottomRight,R.TopLeft);
N:=VectorMultiply2D(D,A);
X:=ScalarDivide2D(VectorSubtract2D(N,D),2);
InflateRect2D(R,X);
end;



procedure RotateRectangleCenter2D(var Rect: T2DRect; Center: T2DVector; Angle: double);
{Rotate the center of specified rectangle around center}
{Does not rotate the individual corner points, only the center}
var C1,C2,Offset: T2DVector;
begin
C1:=ScalarDivide2D(VectorAdd2D(Rect.TopLeft,Rect.BottomRight),2);
C2:=C1;
RotatePoint2D(C2,Center,-Angle);
Offset:=VectorSubtract2D(C2,C1);
OffsetRect2D(Rect,Offset);
end;




function MaxRotatedRect2D(Rect: T2DRect; Center: T2DVector; Angle: double): T2DRect;
{Returns the maximum enclosing rectangle after rotation}
var UL,UR,LL,LR: T2DVector;
var MinX,MinY,MaxX,MaxY: double;
begin
UL:=Rect.TopLeft; LR:=Rect.BottomRight;
LL:=MakeVector2D(Rect.Left,Rect.Bottom);
UR:=MakeVector2D(Rect.Right,Rect.Top);
RotatePoint2D(UL,Center,Angle);
RotatePoint2D(UR,Center,Angle);
RotatePoint2D(LL,Center,Angle);
RotatePoint2D(LR,Center,Angle);

MinX:=Min(Min(UL.X,UR.X), Min(LL.X,LR.X));
MaxX:=Max(Max(UL.X,UR.X), Max(LL.X,LR.X));
MinY:=Min(Min(UL.Y,UR.Y), Min(LL.Y,LR.Y));
MaxY:=Max(Max(UL.Y,UR.Y), Max(LL.Y,LR.Y));
Result:=MakeRect2D(MinX,MinY,MaxX,MaxY);
end;




{------------ 3D Vector math routines --------------------}



function VectorAdd3D(V1,V2: T3DVector): T3DVector;
{Do dot product of two vectors}
begin
Result.X:= V1.X + V2.X;
Result.Y:= V1.Y + V2.Y;
Result.Z:= V1.Z + V2.Z;
end;


function VectorSubtract3D(V1,V2: T3DVector): T3DVector;
{Do vector substract V1-V2}
begin
Result.X:= V1.X - V2.X;
Result.Y:= V1.Y - V2.Y;
Result.Z:= V1.Z - V2.Z;
end;



function VectorMultiply3D(V1,V2: T3DVector): T3DVector;
{Multiply vector components}
begin
Result.X:=V1.X * V2.X;
Result.Y:=V1.Y * V2.Y;
Result.Z:=V1.Z * V2.Z;
end;



function VectorDivide3D(V1,V2: T3DVector): T3DVector;
{Multiply vector components}
begin
Result.X:=V1.X / V2.X;
Result.Y:=V1.Y / V2.Y;
Result.Z:=V1.Z / V2.Z;
end;


function VectorSquare3D(V: T3DVector): T3DVector;
{Square the components of specified vector}
begin
Result:=VectorMultiply3D(V,V);
end;


function VectorSqrRoot3D(V: T3DVector): T3DVector;
{Take square root of the vector's components}
begin
Result.X:=Sqrt(V.X);
Result.Y:=Sqrt(V.Y);
Result.Z:=Sqrt(V.Z);
end;



function VectorFraction3D(V: T3DVector): T3DVector;
{Return fractional part of vector components}
begin
Result.X:=Frac(V.X);
Result.Y:=Frac(V.Y);
Result.Z:=Frac(V.Z);
end;



function DotProduct3D(V1,V2: T3DVector): double;
{Do dot product of two vectors}
begin
Result:=(V1.X * V2.X) + (V1.Y * V2.Y) + (V1.Z * V2.Z);
end;



function CrossProduct3D(V1,V2: T3DVector): T3DVector;
{Do crosss product of two vectors - assume left-hand coordinate system}
begin
Result.X:= ((V1.Y * V2.Z) - (V1.Z * V2.Y));
Result.Y:= -((V1.X * V2.Z) - (V1.Z * V2.X));
Result.Z:= ((V1.X * V2.Y) - (V1.Y * V2.X));
end;



function RightCrossProduct3D(V1,V2: T3DVector): T3DVector;
{Do crosss product of two vectors - assumes right-hand coordinate system}
begin
Result.X:= V1.Y * V2.Z - V1.Z * V2.Y;
Result.Y:= V1.Z * V2.X - V1.X * V2.Z;
Result.Z:= V1.X * V2.Y - V1.Y * V2.X;
end;



function ScalarProduct3D(V: T3DVector; S: double): T3DVector;
{Multiply vector by scalar}
begin
Result.X:=V.X * S;
Result.Y:=V.Y * S;
Result.Z:=V.Z * S;
end;



function ScalarAdd3D(V: T3DVector; S: double): T3DVector;
{Add scalar to vector }
begin
Result.X:=V.X + S;
Result.Y:=V.Y + S;
Result.Z:=V.Z + S;
end;


function ScalarSubtract3D(V: T3DVector; S: double): T3DVector;
{Subtract scalar from vector }
begin
Result.X:=V.X - S;
Result.Y:=V.Y - S;
Result.Z:=V.Z - S;
end;


function ScalarDivide3D(V: T3DVector; S: double): T3DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S;
Result.Y:=V.Y / S;
Result.Z:=V.Z / S;
end;




function VectorABS3D(V: T3DVector): double;
{Find ABS of vector}
begin
Result:=Sqrt(Sqr(V.X) + Sqr(V.Y) + Sqr(V.Z));
end;


function UnitVector3D(V: T3DVector): T3DVector;
{Return unit vector}
var L: double;
begin
L:=VectorABS3D(V);
if L=0.0 then L:=1E-99;
Result.X:=V.X / L;
Result.Y:=V.Y / L;
Result.Z:=V.Z / L;
end;


function VectorAngle3D(V1,V2: T3DVector): double;
{Return angle of two vectors}
var T: double;
begin
T:=VectorABS3D(V1) * VectorABS3D(V2);
if T=0 then Result:=0
else
	begin
	T:=DotProduct3D(V1,V2) / T;
	if T>1 then T:=1 else if T<-1 then T:=-1;
	Result:=ArcCos(T);
	end;
end;


function MakeVector3D(X,Y,Z: double): T3DVector;
{Create a vector from component values }
begin
Result.X:=X;
Result.Y:=Y;
Result.Z:=Z;
end;


function MakeVector3D(V: T2DVector; Z: double): T3DVector;
{Create a vector from from a 2D vector a  }
begin
Result:=MakeVector3D(V.X,V.Y,Z);
end;


function MakeVector3DY(V: T2DVector): T3DVector;
begin
Result:=MakeVector3D(V,V.Y);
end;


function Vector3DTo2D(V: T3DVector): T2DVector;
{Convert a 3D vector to 2D (Using pointer arithmetic) }
begin
Result:=P2DVector(@V)^;
end;



function GetMinimumVector(V1,V2: T3DVector): T3DVector;
{Return minimum components of two vectors}
begin
Result.X:=Min(V1.X,V2.X);
Result.Y:=Min(V1.Y,V2.Y);
Result.Z:=Min(V1.Z,V2.Z);
end;


function GetMaximumVector(V1,V2: T3DVector): T3DVector;
{Return minimum components of two vectors}
begin
Result.X:=Max(V1.X,V2.X);
Result.Y:=Max(V1.Y,V2.Y);
Result.Z:=Max(V1.Z,V2.Z);
end;



function RangeCheckVectorMin(V,Min: T3DVector): T3DVector; overload;
{Ensure vector components are greater or equal to Min Vector}
begin
Result:=V;
if Result.X<Min.X then Result.X:=Min.X;
if Result.Y<Min.Y then Result.Y:=Min.Y;
if Result.Z<Min.Z then Result.Z:=Min.Z;
end;



function RangeCheckVectorMin(V: T3DVector; Min: double): T3DVector;  overload;
{Ensure vector components are greater or equal to Min value}
begin
Result:=RangeCheckVectorMin(V,MakeVector3D(Min,Min,Min));
end;



function RangeCheckVectorMax(V,Max: T3DVector): T3DVector; overload;
{Ensure vector components are less than or equal to Max Vector}
begin
Result:=V;
if Result.X>Max.X then Result.X:=Max.X;
if Result.Y>Max.Y then Result.Y:=Max.Y;
if Result.Z>Max.Z then Result.Z:=Max.Z;
end;



function RangeCheckVectorMax(V: T3DVector; Max: double): T3DVector;  overload;
{Ensure vector components are less than or equal to max value}
begin
Result:=RangeCheckVectorMax(V,MakeVector3D(Max,Max,Max));
end;



procedure ExchangeVectors3D(var V1,V2: T3DVector);
{Swap Vectors V1 and V2}
var T: T3DVector;
begin
T:=V1; V1:=V2; V1:=T;
end;



function Vector3DToString(V: T3DVector; Digits: integer): string; overload
{Return 3d vector as string. Digits past decimal in Digits}
begin
Result:='X: '+FloatToStrF(V.X,ffFixed,18,Digits)+
        ' Y: '+FloatToStrF(V.Y,ffFixed,18,Digits)+
        ' Z: '+FloatToStrF(V.Z,ffFixed,18,Digits);
end;



function Vector3DToString(V: T3DVector; Digits: integer; Separator: char): string; overload;
{Return 3d vector as string. Digits past decimal in Digits}
begin
Result:=FloatToStrF(V.X,ffFixed,18,Digits)+Separator+
	FloatToStrF(V.Y,ffFixed,18,Digits)+Separator+
	FloatToStrF(V.Z,ffFixed,18,Digits);
end;


{----------- 3D Line Routines ------------------------}


function GetDirectedVector3D(P1,P2: T3DVector; Length: double): T3DVector;
{Returns a vector in direction defined by P1 to P2 of specified length }
begin
Result:=VectorSubtract3D(P1,P2);
Result:=UnitVector3D(Result);
Result:=ScalarProduct3D(Result,Length);
end;


procedure TrimStartLine3D(var L: T3DLine; Amount: double);
{Trim start of line by specified amount}
var O: T3DVector;
begin
O:=GetDirectedVector3D(L.P2,L.P1,Amount);
L.P1:=VectorAdd3D(L.P1,O);
end;



procedure TrimEndLine3D(var L: T3DLine; Amount: double);
{Trim end of line by specified amount}
var O: T3DVector;
begin
O:=GetDirectedVector3D(L.P1,L.P2,Amount);
L.P2:=VectorAdd3D(L.P2,O);
end;


function MakeLine3D(P1,P2: T3DVector): T3DLine;
{Return line specified by two vectors}
begin
Result.P1:=P1;
Result.P2:=P2;
end;


{----------- Plane handling procedures ---------------}

procedure TPlane.SetPlane(Azm,Dip: double);
{Set plane based on azimuth and dip }
var PLen: double;
var V1,V2: T3DVector;
begin
Dip:=Dip * Degree2Radian;
Azm:=Azm * Degree2Radian;

PLen:=Cos(Dip);
V1.Z:=Sin(Dip);
V1.Y:=PLen * Cos(Azm);
V1.X:=PLen * Sin(Azm);

V2.Y:=Cos(Azm - 90);
V2.X:=Sin(Azm - 90);
V2.Z:=0;

Con:=CrossProduct3D(V1,V2);
end;


function TPlane.GetPlaneZ(X,Y:double): double;
{Get elevation based on East and North position}
begin
Result:=(-(Con.X*X)-Con.Y*Y)/Con.Z;
end;

constructor TPlane.Create;
begin
inherited Create;
SetPlane(0,0);
end;


procedure TPlane.GetAzmDip(var A,D: double);
{Get Azimuth and Dip angle of specified plane}
var V1,V2: T3DVector;
begin
if (Con.X=0) and (Con.Y=0) then begin A:=0; D:=0; exit; end;
V1.X:=0; V1.Y:=0; V1.Z:=1;
D:=-Radian2Degree * VectorAngle3D(V1,Con);

V2:=CrossProduct3D(Con,V1);
V1.X:=0; V1.Y:=1; V1.Z:=0;
A:=Radian2Degree * VectorAngle3D(V1,V2);
A:=Radian2Degree * ArcTan2(V2.X,V2.Y)-90;
if A<0 then A:=A+360;
end;


procedure TPlane.SetEquation(V: T3DVector);
{Set plane equation based on Azimuth/Dip vector}
begin
Con:=V;
if Con.Z<0 then Con:=ScalarProduct3D(Con,-1);
end;



function TPlane.GetEquation: T3DVector;
{Return the actual plan equations}
begin
Result:=Con;
end;


end.